import sys, os
from struct import pack, unpack
from socket import socket, AF_INET, SOCK_STREAM, SOL_SOCKET, SO_REUSEADDR, SO_ERROR
from socket import error as sock_error

from nitro.async import AsyncOp
from nitro.errors import *

def tcpListen( addr, reactor, callback ) :
    l = TCPListener( reactor, callback )
    l.listen( addr )
    return l

class TCPListener(object) :
    def __init__( self, reactor, callback ) :
        self.reactor = reactor
        self.callback = callback
        self.initialized = False
        self.enabled = False

    def setCallback( self, callback ) :
        self.callback = callback

    def getSock( self ) :
        assert self.initialized
        return self.sock

    def close( self ) :
        assert self.initialized
        self.enable( False )
        self.sock.close()
        self.initialized = False

    def enable( self, flag ) :
        assert self.initialized
        if self.enabled == flag : return
        self.enabled = flag
        if flag :
            self.reactor.addReadCallback( self.sock.fileno(), self._onRead )
        else :
            self.reactor.removeReadCallback( self.sock.fileno() )

    def listen( self, addr ) :
        self.sock = socket( AF_INET, SOCK_STREAM )
        try :
            self.sock.setsockopt( SOL_SOCKET, SO_REUSEADDR, 1 )
            self.sock.bind( addr )
            self.sock.listen( 5 )
        except sock_error :
            self.sock.close()
            raise
        self.initialized = True
        self.enable( True )

    def _onRead( self ) :
        (newSock, address) = self.sock.accept()
        newSock.setblocking( 0 )
        self.callback( newSock )

def tcpConnect( addr, reactor, callback=None ) :
    return TCPConnector().connect( addr, reactor, callback )

class TCPConnector( object ) :
    def __init__( self ) :
        self.errorCode = EINPROGRESS
        self.errorMsg = 'In progress.'
        self.sock = None

    def getError( self ) : return self.errorCode
    def getErrorMsg( self ) : return self.errorMsg
    def getSock( self ) : return self.sock

    def connect( self, addr, reactor, callback=None ) :
        def doConnect() :
            def doNotify( err=0, errMsg='' ) :
                (self.errorCode, self.errorMsg) = (err,errMsg)
                if err == 0 :
                    self.sock = sock
                else :
                    sock.close()
                op.notify( self )

            def addEvents() :
                reactor.addWriteCallback( sock.fileno(), onEvent )
                if (sys.platform == 'win32') and (not reactor.usesWSAAsyncSelect()) :
                    reactor.addExceptionCallback( sock.fileno(), onEvent )

            def removeEvents() :
                reactor.removeWriteCallback( sock.fileno() )
                if (sys.platform == 'win32') and (not reactor.usesWSAAsyncSelect()) :
                    reactor.removeExceptionCallback( sock.fileno() )
                if pollTimer :
                    pollTimer.cancel()

            def doCancel() :
                removeEvents()
                sock.close()

            def onPollTimer() :
                try :
                    err = sock.getsockopt( SOL_SOCKET, SO_ERROR )
                    if err not in (0,EWOULDBLOCK,EAGAIN,EINPROGRESS) :
                        onEvent()
                except sock_error :
                    pass
            
            def onEvent() :
                removeEvents()
                try :
                    err = sock.getsockopt( SOL_SOCKET, SO_ERROR )
                    doNotify( err, os.strerror(err) )
                except sock_error, (err,errMsg) :
                    doNotify( err, errMsg )

            pollTimer = None
            sock = socket( AF_INET, SOCK_STREAM )
            sock.setblocking( 0 )
            try :
                sock.connect( addr )
                doNotify()
            except sock_error, (err,errMsg) :
                if err in (EWOULDBLOCK,EAGAIN,EINPROGRESS) :
                    addEvents()
                    if reactor.usesWSAAsyncSelect() :
                        pollTimer = reactor.addTimer( 1, onPollTimer )
                    op.setCanceler( doCancel )
                    return
                doNotify( err, errMsg )

        timerOp = reactor.callLater( 0, doConnect )
        op = AsyncOp( callback, timerOp.cancel )
        return op

class TCPWriter( object ) :
    def __init__( self, sock, pendingWrite, reactor, timeout=60 ) :
        self.sock = sock
        self.pendingWrite = pendingWrite
        self.reactor = reactor
        self.timeout = timeout
        self.maxWriteChunk = 8192
        self.reactor.addWriteCallback( self.sock.fileno(), self._onWrite )
        self.timerOp = self.reactor.callLater( self.timeout, self._onTimeout )
        self.writeCompleteCallback = None
        self.errorCallback = None
        self.timeoutCallback = None

    def setWriteCompleteCallback( self, writeCompleteCallback ) :
        self.writeCompleteCallback = writeCompleteCallback
    def setErrorCallback( self, errorCallback ) :
        self.errorCallback = errorCallback
    def setTimeoutCallback( self, timeoutCallback ) :
        self.timeoutCallback = timeoutCallback

    def shutdown( self ) :
        self.timerOp.cancel()
        self.reactor.removeWriteCallback( self.sock.fileno() )

    def onWriteComplete( self ) :
        if not self.writeCompleteCallback :
            raise NotImplementedError
        self.writeCompleteCallback()

    def onError( self, err, errMsg ) :
        if not self.errorCallback :
            raise NotImplementedError
        self.errorCallback( err, errMsg )

    def onTimeout( self ) :
        if not self.timeoutCallback :
            raise NotImplementedError
        self.timeoutCallback()

    def _onWrite( self ) :
        try :
            while self.pendingWrite :
                numWritten = self.sock.send( self.pendingWrite[:self.maxWriteChunk] )
                self.pendingWrite = self.pendingWrite[numWritten:]
        except sock_error, (err,errMsg) :
            if err in (EWOULDBLOCK,EAGAIN) : return
            self.reactor.removeWriteCallback( self.sock.fileno() )
            self.timerOp.cancel()
            self.onError( err, errMsg )
            return
        self.reactor.removeWriteCallback( self.sock.fileno() )
        self.timerOp.cancel()
        self.onWriteComplete()

    def _onTimeout( self ) :
        self.reactor.removeWriteCallback( self.sock.fileno() )
        self.onTimeout()

class TCPCloser( TCPWriter ) :
    def __init__( self, sock, pendingWrite, reactor, callback=None ) :
        TCPWriter.__init__( self, sock, pendingWrite, reactor )
        self.op = AsyncOp( callback, self.shutdown )

    def getOp( self ) : return self.op

    def _notify( self ) :
        self.sock.close()
        self.op.notify()
    def onWriteComplete( self ) : self._notify()
    def onError( self, err, errMsg ) : self._notify()
    def onTimeout( self ) : self._notify()

class TCPStream( object ) :
    def __init__( self, sock, reactor ) :
        sock.setblocking( 0 )
        self.sock = sock
        self.reactor = reactor
        self.readEnabled = False
        self.maxReadSize = 0
        self.writeEnabled = False
        self.pendingWrite = ''
        self.closeCallback = None
        self.errorCallback = None
        self.inputCallback = None
        self.writeCompleteCallback = None
        self.shutdownFlag = False

    def getSock( self ) : return self.sock

    def setCloseCallback( self, closeCallback ) :
        self.closeCallback = closeCallback
    def setErrorCallback( self, errorCallback ) :
        self.errorCallback = errorCallback
    def setInputCallback( self, inputCallback ) :
        self.inputCallback = inputCallback
    def setWriteCompleteCallback( self, writeCompleteCallback ) :
        self.writeCompleteCallback = writeCompleteCallback

    def shutdown( self ) :
        assert not self.shutdownFlag
        self.shutdownFlag = True
        if self.readEnabled :
            self.reactor.removeReadCallback( self.sock.fileno() )
            self.readEnabled = False
        if self.writeEnabled :
            self.reactor.removeWriteCallback( self.sock.fileno() )
            self.writeEnabled = False
            self.pendingWrite = ''

    def close( self, deferred=False, callback=None ) :
        assert not self.shutdownFlag
        if not deferred :
            self.shutdown()
            self.sock.close()
            return
        pendingWrite = self.pendingWrite
        self.shutdown()
        return TCPCloser( self.sock, pendingWrite, self.reactor, callback ).getOp()

    def hasShutdown( self ) : return self.shutdownFlag

    def initiateRead( self, maxReadSize ) :
        self.maxReadSize = maxReadSize
        if not self.readEnabled :
            self.reactor.addReadCallback( self.sock.fileno(), self._onRead )
            self.readEnabled = True

    def cancelRead( self ) :
        if self.readEnabled :
            self.reactor.removeReadCallback( self.sock.fileno() )
            self.readEnabled = False

    def writeData( self, data ) :
        if self.writeEnabled :
            assert self.pendingWrite
            self.pendingWrite += data
            return
        self.pendingWrite = data
        self.writeEnabled = True
        self.reactor.addWriteCallback( self.sock.fileno(), self._onWrite )

    def getPendingWrite( self ) :
        return self.pendingWrite

    def _notifyClose( self ) :
        if self.readEnabled :
            self.reactor.removeReadCallback( self.sock.fileno() )
            self.readEnabled = False
        if self.closeCallback :
            self.closeCallback()
        else :
            self.close()

    def _notifyError( self, err, errMsg ) :
        if self.errorCallback :
            self.errorCallback( err, errMsg )
        else :
            self.close()

    def _notifyInput( self, data ) :
        if self.inputCallback :
            self.inputCallback( data )

    def _notifyWriteComplete( self ) :
        if self.writeCompleteCallback :
            self.writeCompleteCallback()

    def _onRead( self ) :
        assert self.readEnabled
        try :
            data = self.sock.recv( self.maxReadSize )
            if not data :
                self._notifyClose()
                return
            self._notifyInput( data )
        except sock_error, (err,errMsg) :
            if err in (EWOULDBLOCK,EAGAIN) : return
            self._notifyError( err, errMsg )

    def _onWrite( self ) :
        assert self.writeEnabled
        assert self.pendingWrite
        try :
            while self.pendingWrite :
                numWritten = self.sock.send( self.pendingWrite )
                self.pendingWrite = self.pendingWrite[numWritten:]
            self.writeEnabled = False
            self.reactor.removeWriteCallback( self.sock.fileno() )
            self._notifyWriteComplete()
        except sock_error, (err,errMsg) :
            if err in (EWOULDBLOCK,EAGAIN) : return
            self._notifyError( err, errMsg )

class TCPMessageStream( object ) :
    def __init__( self, sock, reactor ) :
        self.stream = TCPStream( sock, reactor )
        self.readEnabled = False
        self.maxMsgLength = 65536
        self.readingLength = True
        self.msgLength = 0
        self.buffer = ''
        self.stream.setInputCallback( self._onInput )

        self.getSock = self.stream.getSock
        self.shutdown = self.stream.shutdown
        self.close = self.stream.close
        self.hasShutdown = self.stream.hasShutdown
        self.getPendingWrite = self.stream.getPendingWrite

        self.setCloseCallback = self.stream.setCloseCallback
        self.setErrorCallback = self.stream.setErrorCallback
        self.setWriteCompleteCallback = self.stream.setWriteCompleteCallback

        self.invalidMessageCallback = None
        self.inputCallback = None

    def setInvalidMessageCallback( self, invalidMessageCallback ) :
        self.invalidMessageCallback = invalidMessageCallback

    def setInputCallback( self, inputCallback ) :
        self.inputCallback = inputCallback

    def enableRead( self, enable ) :
        if enable != self.readEnabled :
            self.readEnabled = enable
            if enable :
                if self.readingLength :
                    assert len(self.buffer) < 4
                    self.stream.initiateRead( 4-len(self.buffer)  )
                else :
                    assert len(self.buffer) < self.msgLength
                    self.stream.initiateRead( self.msgLength - len(self.buffer) )
            else :
                self.stream.cancelRead()

    def isReadEnabled( self ) : return self.readEnabled

    def sendMessage( self, data ) :
        msg = pack( '<i%ds' % len(data), len(data), data )
        self.stream.writeData( msg )

    def _onInput( self, data ) :
        assert self.readEnabled
        if self.readingLength :
            assert len(data) + len(self.buffer) <= 4
            self.buffer += data
            if len(self.buffer) == 4 :
                self.msgLength = unpack( '<i', self.buffer )[0]
                self.readingLength = False
                if (self.msgLength <= 0) or (self.msgLength > self.maxMsgLength) :
                    if self.invalidMessageCallback :
                        self.invalidMessageCallback()
                    else :
                        self.close()
                    return
                self.buffer = ''
                self.stream.initiateRead( self.msgLength )
            else :
                self.stream.initiateRead( 4-len(self.buffer) )
        else :
            assert len(data) + len(self.buffer) <= self.msgLength
            self.buffer += data
            if len(self.buffer) == self.msgLength :
                data = self.buffer
                self.buffer = ''
                self.msgLength = 0
                self.readingLength = True
                self.stream.initiateRead( 4 )
                if self.inputCallback :
                    self.inputCallback( data )
            else :
                self.stream.initiateRead( self.msgLength - len(self.buffer) )



#!/usr/bin/env python
#installer.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in installer.py')
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet.task import LoopingCall


import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.packetid as packetid
import lib.settings as settings
import lib.contacts as contacts
import lib.transport_control as transport_control
import lib.diskusage as diskusage
from lib.automat import Automat


import initializer
import p2p_connector
import identity_registrator
import identity_restorer
import install_wizard
import lib.automats as automats

import dhninit
import dhnupdate
import identitypropagate
import webcontrol


_Installer = None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _Installer
    if _Installer is None:
        _Installer = Installer('installer', 'AT_STARTUP', 4)
    if event is not None:
        _Installer.automat(event, arg)
    return _Installer


class Installer(Automat):
    output = {}
    RECOVER_RESULTS = {
        'remote_identity_not_valid':  ('remote Identity is not valid', 'red'),
        'invalid_identity_source':    ('incorrect source of the Identity file', 'red'),
        'invalid_identity_url':       ('incorrect Identity file location', 'red'),
        'remote_identity_bad_format': ('incorrect format of the Identity file', 'red'),
        'incorrect_key':              ('Private Key is not valid', 'red'),
        'idurl_not_exist':            ('Identity URL address not exist or not reachable at this moment', 'blue'),
        'signing_error':              ('unable to sign the local Identity file', 'red'),
        'signature_not_match':        ('remote Identity and Private Key did not match', 'red'),
        'central_failed':             ('unable to connect to the Central server, try again later', 'blue'),
        'success':                    ('account restored!', 'green'), }

    def getOutput(self, state=None):
        if state is None:
            state = self.state
        return self.output.get(state, {})

    def init(self):
        self.flagCmdLine = False
        
    def state_changed(self, oldstate, newstate):
        automats.set_global_state('INSTALL ' + newstate)
        initializer.A('installer.state', newstate)

    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state is 'AT_STARTUP':
            if event == 'init' :
                self.state = 'WHAT_TO_DO?'
                self.flagCmdLine=False
            elif event == 'register-cmd-line' :
                self.state = 'REGISTER'
                self.flagCmdLine=True
                identity_registrator.A('start', arg)
            elif event == 'recover-cmd-line' :
                self.state = 'RECOVER'
                self.flagCmdLine=True
                identity_restorer.A('start', arg)
        #---WHAT_TO_DO?---
        elif self.state is 'WHAT_TO_DO?':
            if event == 'register-selected' :
                self.state = 'INPUT_NAME'
                self.doUpdate(arg)
            elif event == 'recover-selected' :
                self.state = 'LOAD_KEY'
                self.doUpdate(arg)
        #---INPUT_NAME---
        elif self.state is 'INPUT_NAME':
            if event == 'back' :
                self.state = 'WHAT_TO_DO?'
                self.doClearOutput(arg)
                self.doUpdate(arg)
            elif event == 'register-start' and self.isNameValid(arg) :
                self.state = 'REGISTER'
                self.doClearOutput(arg)
                identity_registrator.A('start', arg)
                self.doUpdate(arg)
            elif event == 'register-start' and not self.isNameValid(arg) :
                self.doClearOutput(arg)
                self.doPrintIncorrectName(arg)
                self.doUpdate(arg)
            elif event == 'print' :
                self.doPrint(arg)
                self.doUpdate(arg)
        #---LOAD_KEY---
        elif self.state is 'LOAD_KEY':
            if event == 'back' :
                self.state = 'WHAT_TO_DO?'
                self.doClearOutput(arg)
                self.doUpdate(arg)
            elif event == 'load-from-file' :
                self.doReadKey(arg)
                self.doUpdate(arg)
            elif event == 'paste-from-clipboard' :
                self.doPasteKey(arg)
                self.doUpdate(arg)
            elif event == 'restore-start' :
                self.state = 'RECOVER'
                self.doClearOutput(arg)
                identity_restorer.A('start', arg)
                self.doUpdate(arg)
            elif event == 'print' :
                self.doPrint(arg)
                self.doUpdate(arg)
        #---REGISTER---
        elif self.state is 'REGISTER':
            if event == 'print' :
                self.doPrint(arg)
                self.doUpdate(arg)
            elif ( event == 'identity_registrator.state' and arg is 'READY' ) and not self.flagCmdLine :
                self.state = 'INPUT_NAME'
                self.doUpdate(arg)
            elif ( event == 'identity_registrator.state' and arg in [ 'READY' , 'REGISTERED' ] ) and self.flagCmdLine :
                self.state = 'DONE'
            elif ( event == 'identity_registrator.state' and arg is 'REGISTERED' ) and not self.flagCmdLine :
                self.state = 'AUTHORIZED'
                self.doUpdate(arg)
        #---AUTHORIZED---
        elif self.state is 'AUTHORIZED':
            if event == 'next' :
                self.state = 'WIZARD'
                self.doUpdate(arg)
        #---RECOVER---
        elif self.state is 'RECOVER':
            if event == 'print' :
                self.doPrint(arg)
                self.doUpdate(arg)
            elif ( event == 'identity_restorer.state' and arg is 'READY' ) and not self.flagCmdLine :
                self.state = 'LOAD_KEY'
                self.doUpdate(arg)
            elif ( event == 'identity_restorer.state' and arg is 'DONE' ) or ( ( event == 'identity_restorer.state' and arg is 'READY' ) and self.flagCmdLine ) :
                self.state = 'DONE'
                self.doUpdate(arg)
        #---DONE---
        elif self.state is 'DONE':
            pass
        #---WIZARD---
        elif self.state is 'WIZARD':
            if ( event == 'install_wizard.state' and arg is 'DONE' ) :
                self.state = 'DONE'
                self.doUpdate(arg)

    def isNameValid(self, arg):
        if not misc.ValidUserName(arg):
            return False
        return True

    def isDirSelected(self, arg):
        return arg.get('opendir', '') != '' 

    def isCentralValid(self, arg):
        needed = arg.get('needed', '')
        donated = arg.get('donated', '')
        bandin = arg.get('bandin', '')
        bandout = arg.get('bandout', '')
        customersdir = arg.get('customersdir', '')
        try:
            neededV = float(needed)
            donatedV = float(donated)
            bandinV = float(bandin)
            bandoutV = float(bandout)
        except:
            return False
        if donatedV < settings.MinimumDonatedMb():
            return False
        if bandinV != 0 and bandinV < settings.MinimumBandwidthInLimitKBSec():
            return False 
        if bandoutV != 0 and bandoutV < settings.MinimumBandwidthOutLimitKBSec():
            return False 
        if not os.path.isdir(customersdir):
            return False
        if not os.access(customersdir, os.W_OK):
            return False
        freeBytes, totalBytes = diskusage.GetDriveSpace(customersdir)
        if freeBytes and totalBytes:
            if freeBytes <= float(donatedV * 1024 * 1024):
                return False
        return True

    def isContactsValid(self, arg):
        email = arg.get('email', '').strip()
        phone = arg.get('phone', '').strip()
        fax = arg.get('fax', '').strip()
        text = arg.get('text', '').strip()
        name = arg.get('name', '').strip()
        surname = arg.get('surname', '').strip()
        nickname = arg.get('nickname', '').strip()
        betatester = arg.get('betatester', '').strip()
        if email and not misc.ValidEmail(email):
            return False
        if phone and not misc.ValidPhone(phone):
            return False
        if fax and not misc.ValidPhone(fax):
            return False
        if len(text) > 500:
            return False
        if name and not misc.ValidName(name):
            return False
        if surname and not misc.ValidName(surname):
            return False
        if nickname and not misc.ValidName(nickname):
            return False
        if betatester == 'True' and not email:
            return False 
        return True

    def doClearOutput(self, arg):
        # dhnio.Dprint(4, 'installer.doClearOutput')
        for state in self.output.keys():
            self.output[state] = {'data': [('', 'black')]}

    def doPrint(self, arg):
        # dhnio.Dprint(4, 'installer.doPrint %s %s' % (self.state, str(arg)))
        if not self.output.has_key(self.state):
            self.output[self.state] = {'data': [('', 'black')]}
        if arg is None:
            self.output[self.state]['data'] = [('', 'black')]
        else:
            self.output[self.state]['data'].append(arg)
        # dhnio.Dprint(0, '  [%s]' % text)

    def doPrintIncorrectName(self, arg):
        text, color = ('incorrect user name', 'red') 
        if not self.output.has_key(self.state):
            self.output[self.state] = {'data': [('', 'black')]}
        self.output[self.state]['data'].append((text, color))
        # dhnio.Dprint(0, '  [%s]' % text)

    def doUpdate(self, arg):
        # dhnio.Dprint(4, 'installer.doUpdate')
        reactor.callLater(0, webcontrol.OnUpdateInstallPage)

#    def doPrintCentralError(self, arg):
#        needed = arg.get('needed', '')
#        donated = arg.get('donated', '')
#        bandin = arg.get('bandin', '')
#        bandout = arg.get('bandout', '')
#        customersdir = arg.get('customersdir', '')
#        try:
#            neededV = float(needed)
#        except:
#            self.doPrint(('incorrect needed space value', 'red'))
#            return 
#        try:
#            donatedV = float(donated)
#        except:
#            self.doPrint(('incorrect donated space value', 'red'))
#            return
#        if donatedV < settings.MinimumDonatedMb():
#            self.doPrint(('you must donate at least %d MB' % settings.MinimumDonatedMb(), 'blue'))
#            return 
#        try:
#            bandinV = int(bandin)
#        except:
#            self.doPrint(('incorrect incoming bandwidth limit value', 'red'))
#            return
#        try:
#            bandoutV = int(bandout)
#        except:
#            self.doPrint(('incorrect outgoing bandwidth limit value', 'red'))
#            return
#        if bandinV != 0 and bandinV < settings.MinimumBandwidthInLimitKBSec():
#            self.doPrint(('incoming bandwidth limit is too small', 'red'))
#            return
#        if bandoutV != 0 and bandoutV < settings.MinimumBandwidthOutLimitKBSec():
#            self.doPrint(('outgoing bandwidth limit is too small', 'red'))
#            return
#        if not os.path.isdir(customersdir):
#            self.doPrint(('directory %s not exist' % customersdir, 'red'))
#            return 
#        if not os.access(customersdir, os.W_OK):
#            self.doPrint(('specified folder does not have write permissions', 'red'))
#            return 
#        freeBytes, totalBytes = diskusage.GetDriveSpace(customersdir)
#        dhnio.Dprint(4, 'diskusage.GetDriveSpace returns %s %s' % (str(freeBytes), str(totalBytes)))
#        if freeBytes and totalBytes:
#            if freeBytes <= float(donatedV * 1024 * 1024):
#                self.doPrint(('you do not have enough free space on the disk', 'red'))
#                return 

#    def doCentralSave(self, arg):
#        needed = arg.get('needed', '')
#        donated = arg.get('donated', '')
#        customersdir = arg.get('customersdir', '')
#        bandin = arg.get('bandin', str(settings.DefaultBandwidthInLimit()))
#        bandout = arg.get('bandout', str(settings.DefaultBandwidthOutLimit()))
#        settings.uconfig().set('central-settings.needed-megabytes', needed+'MB')
#        settings.uconfig().set('central-settings.shared-megabytes', donated+'MB')
#        settings.uconfig().set('network.network-send-limit', bandin)
#        settings.uconfig().set('network.network-receive-limit', bandout)
#        settings.uconfig().set('folder.folder-customers', customersdir)
#        settings.uconfig().update()

#    def doDirSave(self, arg):
#        settings.uconfig().set('folder.folder-customers', arg['customersdir'])
#        settings.uconfig().update()

#    def doPrintContactsError(self, arg):
#        email = arg.get('email', '').strip()
#        phone = arg.get('phone', '').strip()
#        fax = arg.get('fax', '').strip()
#        text = arg.get('text', '').strip()
#        name = arg.get('name', '').strip()
#        surname = arg.get('surname', '').strip()
#        nickname = arg.get('nickname', '').strip()
#        betatester = arg.get('betatester', '').strip()
#        if name and not misc.ValidName(name):
#            self.doPrint(('incorrect name entered, please use only letters and dash', 'red'))
#            return
#        if surname and not misc.ValidName(surname):
#            self.doPrint(('incorrect surname entered, please use only letters and dash', 'red'))
#            return
#        if nickname and not misc.ValidName(nickname):
#            self.doPrint(('incorrect nickname entered, please use only letters and dash', 'red'))
#            return
#        if email and not misc.ValidEmail(email):
#            self.doPrint(('incorrect email address', 'red'))
#            return 
#        if phone and not misc.ValidPhone(phone):
#            self.doPrint(('incorrect phone number', 'red'))
#            return 
#        if fax and not misc.ValidPhone(fax):
#            self.doPrint(('incorrect fax number', 'red'))
#            return 
#        if len(text) > 500:
#            self.doPrint(('the text is too long', 'red'))
#            return
#        if betatester == 'True' and not email:
#            self.doPrint(('to be a beta tester please provide your email address, you need to confirm it later', 'red'))
#            return 

#    def doContactsSave(self, arg):
#        settings.uconfig().set('emergency.emergency-email', arg.get('email', '').strip())
#        settings.uconfig().set('emergency.emergency-phone', arg.get('phone', ''))
#        settings.uconfig().set('emergency.emergency-fax', arg.get('fax', ''))
#        settings.uconfig().set('emergency.emergency-text', arg.get('text', ''))
#        settings.uconfig().set('personal.personal-name', arg.get('name', ''))
#        settings.uconfig().set('personal.personal-surname', arg.get('surname', ''))
#        settings.uconfig().set('personal.personal-nickname', arg.get('nickname', ''))
#        settings.uconfig().set('personal.personal-betatester', arg.get('betatester', ''))
#        if arg.get('betatester', '').lower() == 'true':
#            settings.uconfig().set("logs.debug-level", '10')
#            dhnio.SetDebug(10)
#        settings.uconfig().update()

#    def doUpdatesSave(self, arg):
#        shedule = dhnupdate.blank_shedule(arg)
#        settings.uconfig().set('updates.updates-shedule', dhnupdate.shedule_to_string(shedule))
#        settings.uconfig().update()
    
    def doReadKey(self, arg):
        dhnio.Dprint(2, 'installer.doReadKey arg=[%s]' % str(arg))
        src = dhnio.ReadBinaryFile(arg)
        if len(src) > 1024*10:
            self.doPrint(('file is too big for private key', 'red'))
            return

        try:
            lines = src.split('\n')
            idurl = lines[0]
            keysrc = '\n'.join(lines[1:])
            if idurl != nameurl.FilenameUrl(nameurl.UrlFilename(idurl)):
                idurl = ''
                keysrc = src
        except:
            dhnio.DprintException()
            idurl = ''
            keysrc = src

        if not self.output.has_key(self.state):
            self.output[self.state] = {'data': [('', 'black')]}
        self.output[self.state]['idurl'] = idurl
        self.output[self.state]['keysrc'] = keysrc
        
    def doPasteKey(self, arg):
        src = misc.getClipboardText()
        try:
            lines = src.split('\n')
            idurl = lines[0]
            keysrc = '\n'.join(lines[1:])
            if idurl != nameurl.FilenameUrl(nameurl.UrlFilename(idurl)):
                idurl = ''
                keysrc = src
        except:
            dhnio.DprintException()
            idurl = ''
            keysrc = src
        if not self.output.has_key(self.state):
            self.output[self.state] = {'data': [('', 'black')]}
        self.output[self.state]['idurl'] = idurl





import threading

class ThreadQueue( object ) :
    def __init__( self, callback, reactor ) :
        self.callback = callback
        self.reactor = reactor
        self.timerOp = reactor.addTimer( 0.2, self._onTimer )
        self.messages = []
        self.lock = threading.Lock()

    def shutdown( self ) :
        if self.timerOp :
            self.timerOp.cancel()
            self.timerOp = None

    def postMessage( self, msg ) :
        self.lock.acquire()
        try :
            self.messages.append( msg )
        finally :
            self.lock.release()

    def getMessage( self ) :
        self.lock.acquire()
        try :
            if len(self.messages) > 0 :
                msg = self.messages[0]
                del self.messages[0]
                return msg
            else :
                return None
        finally :
            self.lock.release()

    def _onTimer( self ) :
        msg = self.getMessage()
        if msg is not None :
            self.callback( msg )



#!/usr/bin/python
#contact_status.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in contact_status.py')
    

import lib.dhnio as dhnio
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.contacts as contacts
import lib.identitycache as identitycache
import lib.automat as automat
import lib.settings as settings
import lib.commands as commands

if transport_control._TransportCSpaceEnable:
    import lib.transport_cspace as transport_cspace
if transport_control._TransportUDPEnable:
    import lib.transport_udp_session as transport_udp_session    

import ratings


_ContactsStatusDict = {}


#------------------------------------------------------------------------------ 


def init():
    dhnio.Dprint(4, 'contact_status.init')
    transport_control.AddInboxCallback(Inbox)
    transport_control.AddOutboxCallback(Outbox)
    transport_control.AddOutboxPacketStatusFunc(OutboxStatus)
    transport_control.AddWorkItemSentCallbackFunc(FileSent)
#    if transport_control._TransportCSpaceEnable:
#        transport_cspace.SetContactStatusNotifyFunc(CSpaceContactStatus)
#    if transport_control._TransportUDPEnable:
#        transport_udp_session.SetStateChangedCallbackFunc(TransportUDPSessionStateChanged)


def shutdown():
    dhnio.Dprint(4, 'contact_status.shutdown')
    global _ContactsStatusDict
    for A in _ContactsStatusDict.values():
        automat.clear_object(A.index)
    _ContactsStatusDict.clear()
    

def isOnline(idurl):
    if idurl in [None, 'None', '']:
        return False
    global _ContactsStatusDict
    if idurl not in _ContactsStatusDict.keys():
        dhnio.Dprint(6, 'contact_status.isOnline %s is not found, ' % idurl)
    return A(idurl).state == 'CONNECTED'


def isOffline(idurl):
    if idurl in [None, 'None', '']:
        return True
    global _ContactsStatusDict
    if idurl not in _ContactsStatusDict.keys():
        dhnio.Dprint(6, 'contact_status.isOffline %s is not found' % idurl)
    return A(idurl).state == 'OFFLINE'


def hasOfflineSuppliers():
    for idurl in contacts.getSupplierIDs():
        if isOffline(idurl):
            return True
    return False


def countOfflineAmong(idurls_list):
    num = 0
    for idurl in idurls_list:
        if isOffline(idurl):
            num += 1
    return num

def countOnlineAmong(idurls_list):
    num = 0
    for idurl in idurls_list:
        if idurl:
            if isOnline(idurl):
                num += 1
    return num

#------------------------------------------------------------------------------ 

#def check_contacts(contacts_list):
#    for idurl in contacts_list:
#        if idurl:
#            A(idurl, 'check') 


def A(idurl, event=None, arg=None):
    global _ContactsStatusDict
    if not _ContactsStatusDict.has_key(idurl):
        _ContactsStatusDict[idurl] = ContactStatus(idurl, 'status_%s' % nameurl.GetName(idurl), 'OFFLINE', 10)
    if event is not None:
        _ContactsStatusDict[idurl].automat(event, arg)
    return _ContactsStatusDict[idurl]
      

class ContactStatus(automat.Automat):
    timers = {
        'timer-20sec': (20.0, ['PING', 'ACK?']),
        }
    
    def __init__(self, idurl, name, state, debug_level):
        self.idurl = idurl
        self.time_connected = None
        automat.Automat.__init__(self, name, state, debug_level)
        dhnio.Dprint(10, 'contact_status.ContactStatus %s %s %s' % (name, state, idurl))
        
    def state_changed(self, oldstate, newstate):
        dhnio.Dprint(6, '%s : [%s]->[%s]' % (nameurl.GetName(self.idurl), oldstate.lower(), newstate.lower()))
        
    def A(self, event, arg):
        #---CONNECTED---
        if self.state is 'CONNECTED':
            if event == 'outbox-packet' and self.isPingPacket(arg) :
                self.state = 'PING'
                self.AckCounter=0
                self.doRepaint(arg)
            elif event == 'sent-failed' and self.isDataPacket(arg) :
                self.state = 'OFFLINE'
                self.doRepaint(arg)
        #---OFFLINE---
        elif self.state is 'OFFLINE':
            if event == 'outbox-packet' and self.isPingPacket(arg) :
                self.state = 'PING'
                self.AckCounter=0
                self.doRepaint(arg)
            elif event == 'inbox-packet' :
                self.state = 'CONNECTED'
                self.doRememberTime(arg)
                self.doRepaint(arg)
        #---PING---
        elif self.state is 'PING':
            if event == 'sent-done' :
                self.state = 'ACK?'
                self.AckCounter=0
            elif event == 'inbox-packet' :
                self.state = 'CONNECTED'
                self.doRememberTime(arg)
                self.doRepaint(arg)
            elif event == 'file-sent' :
                self.AckCounter+=1
            elif event == 'sent-failed' and self.AckCounter>1 :
                self.AckCounter-=1
            elif event == 'timer-20sec' or ( event == 'sent-failed' and self.AckCounter==1 ) :
                self.state = 'OFFLINE'
                self.doRepaint(arg)
        #---ACK?---
        elif self.state is 'ACK?':
            if event == 'inbox-packet' :
                self.state = 'CONNECTED'
                self.doRememberTime(arg)
                self.doRepaint(arg)
            elif event == 'timer-20sec' :
                self.state = 'OFFLINE'
            elif event == 'outbox-packet' and self.isPingPacket(arg) :
                self.state = 'PING'
                self.AckCounter=0
                self.doRepaint(arg)

    def isPingPacket(self, arg):
        return arg.Command == commands.Identity() and arg.wide is True

    def isDataPacket(self, arg):
        return arg[0].command not in [commands.Identity(), commands.Ack()]

    def doRememberTime(self, arg):
        self.time_connected = time.time()
        
    def doRepaint(self, arg):
        if transport_control.GetContactAliveStateNotifierFunc() is not None:
            transport_control.GetContactAliveStateNotifierFunc()(self.idurl)
 
#------------------------------------------------------------------------------ 

def OutboxStatus(workitem, proto, host, status, error, message):
    if status == 'finished':
        A(workitem.remoteid, 'sent-done', (workitem, proto, host))
    else:
        A(workitem.remoteid, 'sent-failed', (workitem, proto, host))


def Inbox(newpacket, proto, host):
    A(newpacket.OwnerID, 'inbox-packet', (newpacket, proto, host))
    ratings.remember_connected_time(newpacket.OwnerID)
    

def Outbox(outpacket):
    A(outpacket.RemoteID, 'outbox-packet', outpacket)


def FileSent(workitem, args):
    A(workitem.remoteid, 'file-sent', (workitem, args))


def PacketSendingTimeout(remoteID, packetID):
    # dhnio.Dprint(6, 'contact_status.PacketSendingTimeout ' + remoteID)
    A(remoteID, 'sent-timeout', packetID)


def TransportUDPSessionStateChanged(automatindex, oldstate, newstate):
    global _ContactsStatusDict
#    if not _ContactsStatusDict.has_key(idurl):
#        return
    sess = automat.objects().get(automatindex, None)
    if sess is None:
        return
    idurl = sess.remote_idurl
    if idurl is None:
        return
    ident = contacts.getContact(idurl)
    if ident is None:
        return
    address = ident.getProtoContact('udp')
    if address is None:
        return True
    try:
        ip, port = address[6:].split(':')
        address = (ip, int(port))
    except:
        dhnio.DprintException()
        return True
    if sess.remote_address == address or sess.remote_address == identitycache.RemapContactAddress(address):
        A(idurl, 'transport_udp_session.state', newstate)

def CSpaceContactStatus(contact, status):
    idurls = identitycache.GetIDURLsByContact('cspace://'+contact)
    idurl = idurls[0] if len(idurls) > 0 else ''
    dhnio.Dprint(10, 'contact_status.CSpaceContactStatus %s:%s is [%s]' % (nameurl.GetName(idurl), contact, status)) 



import logging
from ncrypt0.x509 import X509Error
from ncrypt0.ssl import SSLConnection, SSLError
from ncrypt0.rsa import RSAKey, RSAError
from nitro.async import AsyncOp
from nitro.tcp import tcpListen
from nitro.ssl import sslAccept, sslAbort, SSLMessageStream
from nitro.bencode import encode, decode, DecodeError
from cspace.network.localip import getLocalIP
from cspace.network.sslutil import makeServerSSLContext
from cspace.network.routerclient import RouterClient, routerAccept
from cspace.network.location import DirectLocation, RoutedLocation,\
        UserLocation, publishUserLocation

logger = logging.getLogger('cspace.network.listener')

#logic for listener:
# bind locally on ADDR_ANY
# get a bunch of router nodes
# register with all of them
# determine localip
# build userlocation object and publish it
# then periodically keep publishing it
# re-publish user location when a router registration connection is lost
# shutdown when losing all router registration connections

#notifications:
# notify first successful location publish
# notify incoming connection after successful ssl handshake
# notify close when all router connections are lost

LOCATION_PUBLISH_TIMEOUT = 5*60

class Dummy : pass

class BaseIntStore( object ) :
    # fetch stored int value
    def get( self ) : raise NotImplementedError
    # save stored int value
    def set( self, value ) : raise NotImplementedError

def _publishLocation( rsaKey, location, updateLevelStore,
        dhtClient, nodeTable, callback=None ) :
    def onPublish( result, updateLevel ) :
        updateLevelStore.set( updateLevel )
        op.notify( result )
    publishOp = publishUserLocation( rsaKey, location,
            updateLevelStore.get(), dhtClient, nodeTable,
            onPublish )
    op = AsyncOp( callback, publishOp.cancel )
    return op

def _registerWithRouter( routerAddr, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        if obj.closed : return
        obj.closed = True
        client.close()
    def onError() :
        if obj.closed : return
        obj.closed = True
        client.close()
        op.notify( None, None, None )
    def checkNotify() :
        if (obj.routerId is not None) and (obj.publicIP is not None) :
            client.enableIncoming( False )
            op.notify( client, obj.routerId, obj.publicIP )
    def onRegister( routerId ) :
        if routerId is None :
            onError()
            return
        obj.routerId = routerId
        checkNotify()
    def onPing( publicAddr ) :
        if publicAddr is None :
            onError()
            return
        obj.publicIP = publicAddr[0]
        checkNotify()
    def onConnect( result ) :
        if not result :
            onError()
            return
        client.callPing( onPing )
        client.callRegister( onRegister, incomingCallback=None )
    obj.closed = False
    obj.routerId = None
    obj.publicIP = None
    client = RouterClient( routerAddr, reactor )
    client.connect( onConnect )
    op = AsyncOp( callback, doCancel )
    return op

def _registerWithRouters( routerAddrList, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        for regOp in obj.registerOps.keys() :
            regOp.cancel()
        obj.registerOps.clear()
        for client,rid,ip in obj.result :
            client.close()
        del obj.result[:]
    def checkNotify() :
        if obj.registerOps :
            return False
        if obj.timerOp :
            obj.timerOp.cancel()
            obj.timerOp = None
        op.notify( obj.result )
        return True
    def onTimer() :
        obj.timerOp = None
        for regOp in obj.registerOps.keys() :
            regOp.cancel()
        obj.registerOps.clear()
        checkNotify()
    def onRegister( regOp, client, routerId, publicIP ) :
        del obj.registerOps[regOp]
        if client is not None :
            obj.result.append( (client,routerId,publicIP) )
        if checkNotify() :
            return
        if (obj.timerOp is not None) and (len(obj.result) == 1) :
            obj.timerOp = reactor.callLater( 10, onTimer )
    obj.registerOps = {}
    obj.timerOp = None
    obj.result = []
    def doRegister( routerAddr ) :
        regOp = _registerWithRouter( routerAddr, reactor,
                lambda x,y,z : onRegister(regOp,x,y,z) )
        obj.registerOps[regOp] = 1
    for routerAddr in routerAddrList :
        doRegister( routerAddr )
    op = AsyncOp( callback, doCancel )
    return op

def _getMajority( values ) :
    assert values
    freq = {}
    for v in values :
        freq[v] = freq.get(v,0) + 1
    items = [(v,k) for k,v in freq.items()]
    items.sort()
    return items[-1][1]

def _initializeListen( rsaKey, updateLevelStore, dhtClient,
        nodeTable, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        if obj.publishOp :
            obj.publishOp.cancel()
            obj.publishOp = None
        for client,rid,ip in obj.routerList :
            client.close()
        del obj.routerList[:]
        if obj.registerOp :
            obj.registerOp.cancel()
            obj.registerOp = None
        obj.listener.close()
        obj.listener = None
    def onError() :
        doCancel()
        op.notify( None )
    def onPublish( result ) :
        obj.publishOp = None
        if not result :
            onError()
            return
        data = (obj.listener,obj.routerList,obj.publicIP,\
                obj.localIP,obj.localPort)
        op.notify( data )
    def getLocation() :
        directLocations = [DirectLocation((obj.localIP,obj.localPort))]
        routedLocations = [RoutedLocation(rc.getRouterAddr(),rid)\
                for rc,rid,ip in obj.routerList]
        location = UserLocation( directLocations, routedLocations,
                obj.publicIP )
        return location
    def onRegister( routerList ) :
        obj.registerOp = None
        if not routerList :
            onError()
            return
        obj.routerList = routerList
        obj.localIP = getLocalIP()
        if obj.localIP is None :
            onError()
            return
        ipList = [ip for client,rid,ip in obj.routerList]
        obj.publicIP = _getMajority( ipList )
        obj.publishOp = _publishLocation( rsaKey, getLocation(),
                updateLevelStore, dhtClient, nodeTable, onPublish )
    obj.listener = tcpListen( ('',0), reactor, None )
    obj.listener.enable( False )
    obj.localPort = obj.listener.getSock().getsockname()[1]
    obj.registerOp = _registerWithRouters( nodeTable.getLiveNodes(),
            reactor, onRegister )
    obj.routerList = []
    obj.publishOp = None
    op = AsyncOp( callback, doCancel )
    return op

def _sslHandshake( sock, sslContext, reactor, callback=None ) :
    def doCancel() :
        acceptOp.cancel()
        sslAbort( sslConn )
    def onSSLAccept( err ) :
        global logger
        if err is not None :
            sslAbort( sslConn )
            op.notify( None )
            return
        try :
            peerCert = sslConn.getPeerCertificate()
            peerKey = RSAKey()
            peerKey.fromPKey_PublicKey(peerCert.get_pubkey())
            CN = peerCert.get_subject().CN
            if CN.count('@'):
                peerName, peerKeyID = CN.split('@')
            else:
                peerName, peerKeyID = CN, ''
        except (SSLError,X509Error,RSAError) :
            logger.exception('ssl accept error')
            sslAbort( sslConn )
            op.notify( None )
            return
        data = (sslConn,peerKey,peerName,peerKeyID)
        # logger.info('onSSLAccept SUCCESSFUL with %s' % str(peerName))
        op.notify( data )
    sslConn = SSLConnection( sslContext, sock )
    sslConn.setAcceptState()
    acceptOp = sslAccept( sslConn, reactor, onSSLAccept )
    op = AsyncOp( callback, doCancel )
    return op

def _readServiceName( sock, sslContext, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        if obj.handshakeOp :
            obj.handshakeOp.cancel()
            obj.handshakeOp = None
        if obj.stream :
            obj.stream.close()
            obj.stream = None
    def onError() :
        doCancel()
        op.notify( None )
    def onClose( *args ) :
        onError()
    def onInput( data ) :
        try :
            serviceName = decode( data )
            assert type(serviceName) is str
            assert len(serviceName) > 0
        except :
            onError()
            return
        obj.stream.shutdown()
        obj.stream = None
        op.notify( (obj.sslConn,obj.peerKey,obj.peerName,obj.peerKeyID,serviceName) )
    def onHandshake( data ) :
        obj.handshakeOp = None
        if data is None :
            onError()
            return
        obj.sslConn,obj.peerKey,obj.peerName,obj.peerKeyID = data
        obj.stream = SSLMessageStream( obj.sslConn, reactor )
        obj.stream.setCloseCallback( onClose )
        obj.stream.setErrorCallback( onClose )
        obj.stream.setInvalidMessageCallback( onClose )
        obj.stream.setInputCallback( onInput )
        obj.stream.enableRead( True )
    obj.handshakeOp = _sslHandshake( sock, sslContext, reactor, onHandshake )
    obj.stream = None
    op = AsyncOp( callback, doCancel )
    return op

def acceptIncoming( acceptFlag, sslConn, reactor, callback=None ) :
    def doCancel() :
        stream.close()
    def onError( *args ) :
        doCancel()
        op.notify( None )
    def onWriteComplete() :
        stream.shutdown()
        op.notify( sslConn )
    stream = SSLMessageStream( sslConn, reactor )
    stream.setCloseCallback( onError )
    stream.setErrorCallback( onError )
    stream.setInvalidMessageCallback( onError )
    stream.setInputCallback( onError )
    stream.setWriteCompleteCallback( onWriteComplete )
    if acceptFlag :
        data = 0
    else :
        data = -1
    stream.sendMessage( encode(data) )
    op = AsyncOp( callback, doCancel )
    return op

class UserListener( object ) :
    (STOPPED,STARTING,STARTED,STOPPING) = range(4)
    def __init__( self, userName, keyId, rsaKey, updateLevelStore,
            dhtClient, nodeTable, reactor ) :
        self.userName = userName
        self.keyId = keyId
        self.rsaKey = rsaKey
        self.updateLevelStore = updateLevelStore
        self.dhtClient = dhtClient
        self.nodeTable = nodeTable
        self.reactor = reactor
        self.state = self.STOPPED
        self.closeCallback = None
        self.sslContext = makeServerSSLContext( userName, keyId, rsaKey, self.clientCertVerifyCallback )
        self.incomingCallback = None

        self.initOp = None
        self.tcpListener = None
        self.routerList = None
        self.publicIP = None
        self.localIP = None
        self.localPort = None
        self.publishOp = None
        self.otherOps = {}
        
    def clientCertVerifyCallback(self, conn, cert, errno, depth, preverify_ok):
        return preverify_ok        

    def setIncomingCallback( self, incomingCallback ) :
        self.incomingCallback = incomingCallback

    def setCloseCallback( self, closeCallback ) :
        self.closeCallback = closeCallback

    def getPublicIP( self ) : return self.publicIP

    def _shutdownInternal( self ) :
        while self.otherOps :
            op,x = self.otherOps.popitem()
            op.cancel()
        if self.publishOp is not None :
            self.publishOp.cancel()
            self.publishOp = None
        if self.tcpListener is not None :
            self.tcpListener.close()
            self.tcpListener = None
        if self.routerList is not None :
            for rc,rid,ip in self.routerList :
                rc.close()
            self.routerList = None
        self.publicIP = None
        self.localIP = None
        self.localPort = None
        if self.initOp is not None :
            self.initOp.cancel()
            self.initOp = None

    def close( self ) :
        self._shutdownInternal()
        self.state = self.STOPPED

    def closeNotify( self ) :
        self.close()
        if self.closeCallback :
            self.closeCallback()

    def start( self, startCallback ) :
        assert self.state == self.STOPPED
        self.state = self.STARTING
        def onError() :
            self.state = self.STOPPED
            startCallback( False )
        def onSuccess() :
            self.state = self.STARTED
            startCallback( True )
        def enableRouter( rc ) :
            rc.setIncomingCallback( lambda cid : self._onIncomingRouted(rc,cid) )
            rc.setCloseCallback( lambda : self._onRouterClose(rc) )
            rc.enableIncoming( True )
        def onInit( result ) :
            self.initOp = None
            if result is None :
                onError()
                return
            self.tcpListener,self.routerList,self.publicIP,\
                    self.localIP,self.localPort = result
            self.tcpListener.setCallback( self._onIncomingTCP )
            self.tcpListener.enable( True )
            for rc,rid,ip in self.routerList :
                enableRouter( rc )
            self._initPublishTimer()
            onSuccess()
        self.initOp = _initializeListen( self.rsaKey,
                self.updateLevelStore, self.dhtClient,
                self.nodeTable, self.reactor, onInit )

    def _initPublishTimer( self ) :
        def onDummyPing( publicAddr ) :
            pass
        def getLocation() :
            directLocations = [DirectLocation((self.localIP,self.localPort))]
            routedLocations = [RoutedLocation(rc.getRouterAddr(),rid)\
                    for rc,rid,ip in self.routerList]
            location = UserLocation( directLocations, routedLocations,
                    self.publicIP )
            return location
        def onPublish( result ) :
            self.publishOp = self.reactor.callLater( timeout, onTimer )
        def onTimer() :
            self.publishOp = _publishLocation( self.rsaKey,
                    getLocation(), self.updateLevelStore,
                    self.dhtClient, self.nodeTable, onPublish )
            for rc,rid,ip in self.routerList :
                rc.callPing( onDummyPing )
        timeout = LOCATION_PUBLISH_TIMEOUT
        self.publishOp = self.reactor.callLater( timeout, onTimer )

    def _onRouterClose( self, routerClient ) :
        assert self.state == self.STARTED
        index = -1
        for i,router in enumerate(self.routerList) :
            if router[0] is routerClient :
                index = i
                break
        assert index >= 0
        del self.routerList[index]
        if not self.routerList :
            self.closeNotify()

    def _onIncomingRouted( self, routerClient, connectionId ) :
        assert self.state == self.STARTED
        def onAccept( sock ) :
            del self.otherOps[acceptOp]
            if sock is None :
                return
            self._onIncomingTCP( sock )
        acceptOp = routerAccept( routerClient.getRouterAddr(),
                connectionId, self.reactor, onAccept )
        self.otherOps[acceptOp] = 1

    def _onIncomingTCP( self, sock ) :
        assert self.state == self.STARTED
        def onRead( data ) :
            del self.otherOps[op]
            if data is None :
                return
            sslConn,peerKey,peerName,peerKeyID,serviceName = data
            if self.incomingCallback is None :
                sslAbort( sslConn )
                return
            self.incomingCallback( sslConn, peerKey, peerName, peerKeyID, serviceName )
        op = _readServiceName( sock, self.sslContext,
                self.reactor, onRead )
        self.otherOps[op] = 1

    def stop( self, stopCallback ) :
        assert self.state == self.STARTED
        self.state = self.STOPPING
        self._shutdownInternal()
        def onPublish( result ) :
            del self.otherOps[stopOp]
            self.state = self.STOPPED
            stopCallback( result )
        emptyLocation = UserLocation( [], [], '' )
        stopOp = _publishLocation( self.rsaKey, emptyLocation,
                self.updateLevelStore, self.dhtClient,
                self.nodeTable, onPublish )
        self.otherOps[stopOp] = 1



#!/usr/bin/python
#backup.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#  This interfaces between a pipe from something like tar and the twisted code
#    for rest of DataHaven.NET
#  We see how many packets are waiting to be sent,
#    and if it is not too many, and we can make more, we make some more.
#
#  Main idea:
#     1) When a backup is started a backup object is created
#     2) We get a file descriptor for the process creating the tar archive
#     3) We always use select/poll before reading so we never block
#     4) We also poll to see if more needed.
#     5) We number/name blocks so can be sure what is what when we read back later
#     6) We call raidmake to split block and make parities
#     7) We put parts into dhnpackets and give these to transport_control


import os
import sys
import time
import cStringIO
import gc


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backup.py')

from twisted.internet import threads
from twisted.internet.defer import Deferred, maybeDeferred


import lib.dhnio as dhnio
import lib.misc as misc
import lib.dhnpacket as dhnpacket
import lib.contacts as contacts
import lib.commands as commands
import lib.settings as settings
import lib.packetid as packetid
import lib.nonblocking as nonblocking
import lib.eccmap as eccmap
import lib.dhncrypto as dhncrypto
import lib.tmpfile as tmpfile
import lib.automat as automat
# import lib.automats as automats


import data_sender
import fire_hire


import raidmake
import dhnblock
import events
import backup_db
import backups


#-------------------------------------------------------------------------------

class backup(automat.Automat):
    timers = {'timer-01sec':    (0.1, ['RUN', 'READ']),} 
    
    def __init__(self, backupID, pipe, blockSize=None, resultDeferred=None):
        self.backupID = backupID
        self.eccmap = eccmap.Current()
        self.pipe = pipe
        self.blockSize = blockSize
        if self.blockSize is None:
            self.blockSize = self.eccmap.nodes() * settings.getBackupBlockSize()
        self.ask4abort = False
        self.stateEOF = False
        self.stateReading = False
        self.currentBlockData = cStringIO.StringIO()
        self.currentBlockSize = 0
        self.blockNumber = 0
        self.dataSent = 0
        self.blocksSent = 0
        self.closed = False
        self.result = ''
        if resultDeferred is None:
            self.resultDefer = Deferred()
        else:
            self.resultDefer = resultDeferred
        self.packetResultCallback = None
        self.sendingStats = {}
        automat.Automat.__init__(self, 'backup', 'AT_STARTUP', 10)
        self.automat('init')
        events.info('backup', '%s started' % self.backupID)
        dhnio.Dprint(6, 'backup.__init__ %s %s %d' % (self.backupID, self.eccmap, self.blockSize,))

    def abort(self):
        dhnio.Dprint(4, 'backup.abort id='+str(self.backupID))
        self.ask4abort = True
        self.result = 'abort'

        
    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state is 'AT_STARTUP':
            if event == 'init' :
                self.state = 'RUN'
        #---RUN---
        elif self.state is 'RUN':
            if event == 'timer-01sec' and self.isAborted(arg) :
                self.state = 'ABORTED'
                self.doClose(arg)
                self.doReport(arg)
                self.doDestroyMe(arg)
            elif event == 'timer-01sec' and not self.isAborted(arg) :
                self.state = 'READ'
        #---READ---
        elif self.state is 'READ':
            if event == 'timer-01sec' and self.isPipeReady(arg) and not self.isEOF(arg) and not self.isReadingNow(arg) and not self.isBlockReady(arg) :
                self.doRead(arg)
            elif event == 'timer-01sec' and not self.isReadingNow(arg) and ( self.isBlockReady(arg) or self.isEOF(arg) ) :
                self.state = 'BLOCK'
                self.doBlock(arg)
        #---BLOCK---
        elif self.state is 'BLOCK':
            if event == 'block-ready' :
                self.state = 'RAID'
                self.doRaid(arg)
        #---RAID---
        elif self.state is 'RAID':
            if event == 'raid-done' and not self.isEOF(arg) :
                self.state = 'RUN'
                self.doBlockReport(arg)
                data_sender.A('new-data')
                self.doNewBlock(arg)
            elif event == 'raid-done' and self.isEOF(arg) :
                self.state = 'DONE'
                self.doBlockReport(arg)
                data_sender.A('new-data')
                self.doClose(arg)
                self.doReport(arg)
                self.doDestroyMe(arg)
        #---DONE---
        elif self.state is 'DONE':
            pass
        #---ABORTED---
        elif self.state is 'ABORTED':
            pass

    def isAborted(self, arg):
        return self.ask4abort
         
    def isPipeReady(self, arg):
        return self.pipe is not None and self.pipe.state() in [nonblocking.PIPE_CLOSED, nonblocking.PIPE_READY2READ]
    
    def isBlockReady(self, arg):
        return self.currentBlockSize >= self.blockSize
    
    def isEOF(self, arg):
        return self.stateEOF
    
    def isReadingNow(self, arg):
        return self.stateReading

    def doClose(self, arg):
        self.closed = True
        
    def doDestroyMe(self, arg):
        self.currentBlockData.close()
        del self.currentBlockData
        # automats.get_automats_by_index().pop(self.index)
        automat.objects().pop(self.index)
        reactor.callLater(0, backup_db.RemoveRunningBackupObject, self.backupID)
        collected = gc.collect()
        dhnio.Dprint(6, 'backup.doDestroyMe collected %d objects' % collected)

    def doReport(self, arg):
        if self.result == '':
            self.resultDefer.callback(self.backupID)
            events.info('backup', '%s done successfully' % self.backupID)
        elif self.result == 'abort':  
            self.resultDefer.callback(self.backupID+' abort')
            events.info('backup', '%s aborted' % self.backupID)
        else:
            self.resultDefer.errback(self.backupID)
            dhnio.Dprint(1, 'backup.doReport ERROR %s result is [%s]' % (self.backupID, self.result))
            events.info('backup', '%s failed: %s' % (self.backupID, self.result))

    def doRead(self, arg):
        def readChunk():
            size = self.blockSize - self.currentBlockSize
            if size < 0:
                dhnio.Dprint(1, "backup.readChunk ERROR eccmap.nodes=" + str(self.eccmap.nodes()))
                dhnio.Dprint(1, "backup.readChunk ERROR blockSize=" + str(self.blockSize))
                dhnio.Dprint(1, "backup.readChunk ERROR currentBlockSize=" + str(self.currentBlockSize))
                raise Exception('size < 0, blockSize=%s, currentBlockSize=%s' % (self.blockSize, self.currentBlockSize))
                return ''
            elif size == 0:
                return ''
            if self.pipe is None:
                raise Exception('backup.pipe is None')
                return ''
            if self.pipe.state() == nonblocking.PIPE_CLOSED:
                dhnio.Dprint(4, 'backup.readChunk the state is PIPE_CLOSED !!!!!!!!!!!!!!!!!!!!!!!!')
                return ''
            if self.pipe.state() == nonblocking.PIPE_READY2READ:
                newchunk = self.pipe.recv(size)
                if newchunk == '':
                    dhnio.Dprint(4, 'backup.readChunk pipe.recv() returned empty string')
                return newchunk
            dhnio.Dprint(1, "backup.readChunk ERROR pipe.state=" + str(self.pipe.state()))
            raise Exception('backup.pipe.state is ' + str(self.pipe.state()))
            return ''
        def readDone(data):
            self.currentBlockData.write(data)
            self.currentBlockSize += len(data)
            self.stateReading = False
            if data == '':
                self.stateEOF = True
            #dhnio.Dprint(12, 'backup.readDone %d bytes' % len(data))
        self.stateReading = True
        maybeDeferred(readChunk).addCallback(readDone)

    def doBlock(self, arg):
        def _doBlock():
            # dhnio.Dprint(12, 'backup.doBlock blockNumber=%d size=%d atEOF=%s' % (self.blockNumber, self.currentBlockSize, self.stateEOF))
            src = self.currentBlockData.getvalue()
            block = dhnblock.dhnblock(
                misc.getLocalID(),
                self.backupID,
                self.blockNumber,
                dhncrypto.NewSessionKey(),
                dhncrypto.SessionKeyType(),
                self.stateEOF,
                src,)
            del src
            return block
        maybeDeferred(_doBlock).addCallback(
            lambda block: self.automat('block-ready', block),)

    def doRaid(self, arg):
        newblock = arg
        dhnio.Dprint(8, 'backup.doRaid block=%d size=%d eof=%s ' % (
            self.blockNumber, self.currentBlockSize, str(self.stateEOF),))
        fileno, filename = tmpfile.make('raid')
        serializedblock = newblock.Serialize()
        blocklen = len(serializedblock)
        os.write(fileno, str(blocklen) + ":" + serializedblock)
        os.close(fileno)
        threads.deferToThread(raidmake.raidmake, 
                              filename, 
                              self.eccmap.name, 
                              self.backupID, 
                              self.blockNumber).addBoth(
                                  lambda outDir: self.automat('raid-done', newblock))
        del serializedblock

    def doBlockReport(self, arg):
        newblock = arg
        for supplierNum in range(self.eccmap.nodes()):
            for DataOrParity in ('Data', 'Parity'):
                backups.LocalFileReport(None, newblock.BackupID, newblock.BlockNumber, supplierNum, DataOrParity)

    def doNewBlock(self, arg):
        self.dataSent += self.currentBlockSize
        self.blocksSent += 1
        #self.currentBlockData = ''
        self.currentBlockData.close()
        del self.currentBlockData
        self.currentBlockData = cStringIO.StringIO()
        self.currentBlockSize = 0
        self.blockNumber += 1
        if self.packetResultCallback is not None:
            self.packetResultCallback(self.backupID, None)






import pycurl
from nitro.async import AsyncOp

def curlPerform( curlHandle, reactor, callback=None ) :
    return CurlPerformer(curlHandle,reactor,callback).getOp()

class CurlPerformer( object ) :
    def __init__( self, curlHandle, reactor, callback=None ) :
        self.reactor = reactor
        self.cm = pycurl.CurlMulti()
        self.c = curlHandle
        self.cm.add_handle( self.c )
        self.fdsets = ( [], [], [] )
        self.timerOp = self.reactor.callLater( 0, self._onInitialTimer )
        self.op = AsyncOp( callback, self._doCancel )

    def getOp( self ) : return self.op

    def _onInitialTimer( self ) :
        self.timerOp = self.reactor.addTimer( 1, self._onProcess )
        self._onProcess()

    def _addEvents( self ) :
        self.fdsets = self.cm.fdset()
        r = self.reactor
        for fd in self.fdsets[0] :
            r.addReadCallback( fd, self._onProcess )
        for fd in self.fdsets[1] :
            r.addWriteCallback( fd, self._onProcess )
        for fd in self.fdsets[2] :
            r.addExceptionCallback( fd, self._onProcess )

    def _removeEvents( self ) :
        r = self.reactor
        for fd in self.fdsets[0] :
            r.removeReadCallback( fd )
        for fd in self.fdsets[1] :
            r.removeWriteCallback( fd )
        for fd in self.fdsets[2] :
            r.removeExceptionCallback( fd )

    def _doCancel( self ) :
        self.timerOp.cancel()
        self.timerOp = None
        self._removeEvents()
        self.cm.remove_handle( self.c )
        self.cm.close()

    def _doNotify( self, errCode ) :
        self.timerOp.cancel()
        self.timerOp = None
        self.cm.remove_handle( self.c )
        self.cm.close()
        self.op.notify( errCode )

    def _onProcess( self ) :
        self._removeEvents()
        while 1 :
            ret,numHandles = self.cm.perform()
            if ret != pycurl.E_CALL_MULTI_PERFORM :
                break
        numQ, okList, errList = self.cm.info_read()
        assert numQ == 0
        assert len(okList)+len(errList) <= 1
        if len(okList) > 0 :
            self._doNotify( 0 )
        elif len(errList) > 0 :
            c,err,errMsg = errList[0]
            self._doNotify( err )
        else :
            self._addEvents()

class HttpRequest( object ) :
    def __init__( self, reactor ) :
        self.reactor = reactor
        self.c = pycurl.Curl()
        self.c.setopt( pycurl.FOLLOWLOCATION, 1 )
        self.c.setopt( pycurl.MAXREDIRS, 5 )
        self.c.setopt( pycurl.NOSIGNAL, 1 )
        self.headers = []

    def addHeader( self, header ) :
        self.headers.append( header )

    def _request( self, url, callback ) :
        self.c.setopt( pycurl.URL, url )
        if self.headers :
            self.c.setopt( pycurl.HTTPHEADER, self.headers )
        self.data = []
        self.c.setopt( pycurl.WRITEFUNCTION, self.data.append )
        def doCancel() :
            curlOp.cancel()
            self.c.close()
        def onDone( errCode ) :
            if errCode != 0 :
                code,data = -1,None
            else :
                code = self.c.getinfo( pycurl.RESPONSE_CODE )
                data = ''.join( self.data )
            self.c.close()
            op.notify( code, data )
        curlOp = curlPerform( self.c, self.reactor, onDone )
        op = AsyncOp( callback, doCancel )
        return op

    def get( self, url, callback=None ) :
        return self._request( url, callback )

    def post( self, url, postData, callback=None ) :
        self.c.setopt( pycurl.POST, 1 )
        self.c.setopt( pycurl.POSTFIELDS, postData )
        self.c.setopt( pycurl.POSTFIELDSIZE, len(postData) )
        hasExpect = False
        for h in self.headers :
            if h.startswith('Expect:') : hasExpect = True
        if not hasExpect :
            self.addHeader( 'Expect:' )
        return self._request( url, callback )

if __name__ == '__main__' :
    import sys, logging
    logging.getLogger().addHandler( logging.StreamHandler() )
    from nitro.selectreactor import SelectReactor
    reactor = SelectReactor()
    url = sys.argv[1]
    def onHttpGet( returnCode, data ) :
        print 'returnCode = %d' % returnCode
        if data is not None :
            print 'len(data) = %d' % len(data)
            sys.stdout.write( data )
            sys.stdout.flush()
        reactor.stop()
    HttpRequest( reactor ).get( url, onHttpGet )
    reactor.run()



from nitro.async import AsyncOp
from nitro.tcp import tcpConnect
from cspace.util.validate import validateInetAddress
from cspace.util.rpc import RPCConnection
import logging

logger = logging.getLogger('cspace.network.routerclient')

class Dummy : pass

def _doRPCCall( rpcConn, msg, args, callback=None ) :
    def onResult( err, result ) :
        if err < 0 :
            op.notify( -1, None )
            return
        try :
            assert type(result) is list
            assert len(result) >= 1
            remoteErr = result.pop( 0 )
            assert type(remoteErr) is int
        except :
            op.notify( -1, None )
            return
        if remoteErr < 0 :
            op.notify( -1, None )
            return
        op.notify( 0, result )
    rpcOp = rpcConn.request( [msg]+args, onResult )
    op = AsyncOp( callback, rpcOp.cancel )
    return op

def routerConnect( routerAddr, routerId, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        if obj.op : obj.op.cancel()
        if obj.rpcConn : obj.rpcConn.close()
    def onError() :
        doCancel()
        op.notify( None )
    def onRouterConnect( err, result ) :
        obj.op = None
        if err < 0 :
            onError()
            return
        sock = obj.rpcConn.getSock()
        obj.rpcConn.shutdown()
        op.notify( sock )
    def onTCPConnect( connector ) :
        obj.op = None
        if connector.getError() != 0 :
            onError()
            return
        obj.rpcConn = RPCConnection( connector.getSock(), reactor )
        obj.rpcConn.setCloseCallback( onError )
        obj.op = _doRPCCall( obj.rpcConn, 'Connect', [routerId], onRouterConnect )
    obj.op = tcpConnect( routerAddr, reactor, onTCPConnect )
    op = AsyncOp( callback, doCancel )
    obj.rpcConn = None
    return op

def routerAccept( routerAddr, connectionId, reactor, callback=None ) :
    obj = Dummy()
    def doCancel() :
        if obj.op : obj.op.cancel()
        if obj.rpcConn : obj.rpcConn.close()
    def onError() :
        doCancel()
        op.notify( None )
    def onRouterAccept( err, result ) :
        obj.op = None
        if err < 0 :
            onError()
            return
        sock = obj.rpcConn.getSock()
        obj.rpcConn.shutdown()
        op.notify( sock )
    def onTCPConnect( connector ) :
        obj.op = None
        if connector.getError() != 0 :
            onError()
            return
        obj.rpcConn = RPCConnection( connector.getSock(), reactor )
        obj.rpcConn.setCloseCallback( onError )
        obj.op = _doRPCCall( obj.rpcConn, 'Accept', [connectionId],
                onRouterAccept )
    obj.op = tcpConnect( routerAddr, reactor, onTCPConnect )
    op = AsyncOp( callback, doCancel )
    obj.rpcConn = None
    return op

class RouterClient( object ) :
    (CLOSED,CONNECTING,CONNECTED,REGISTERING,REGISTERED) = range(5)
    def __init__( self, routerAddr, reactor ) :
        self.routerAddr = routerAddr
        self.reactor = reactor
        self.state = self.CLOSED
        self.connectOp = None
        self.rpcConn = None
        self.pingOps = {}
        self.registerOp = None
        self.closeCallback = None

    def setCloseCallback( self, closeCallback ) :
        self.closeCallback = closeCallback

    def close( self ) :
        if self.state == self.CONNECTING :
            self.connectOp.cancel()
            self.connectOp = None
        else :
            for pingOp in self.pingOps.keys() :
                pingOp.cancel()
            self.pingOps.clear()
            if self.state == self.REGISTERING :
                self.registerOp.cancel()
                self.registerOp = None
            if self.state in (self.CONNECTED,self.REGISTERING,self.REGISTERED) :
                self.rpcConn.close()
                self.rpcConn = None
        self.state = self.CLOSED

    def enableIncoming( self, flag ) :
        assert self.state == self.REGISTERED
        self.rpcConn.allowIncoming( flag )

    def setIncomingCallback( self, incomingCallback ) :
        assert self.state == self.REGISTERED
        self.incomingCallback = incomingCallback

    def getRouterAddr( self ) : return self.routerAddr

    def _onClose( self ) :
        self.close()
        if self.closeCallback :
            self.closeCallback()

    def connect( self, connectCallback ) :
        assert self.state == self.CLOSED
        def onTCPConnect( connector ) :
            self.connectOp = None
            if connector.getError() != 0 :
                self.state = self.CLOSED
                connectCallback( False )
                return
            self.rpcConn = RPCConnection( connector.getSock(), self.reactor )
            self.rpcConn.setCloseCallback( self._onClose )
            self.state = self.CONNECTED
            connectCallback( True )
        self.connectOp = tcpConnect( self.routerAddr, self.reactor, onTCPConnect )
        self.state = self.CONNECTING

    def callPing( self, pingCallback ) :
        assert self.state in (self.CONNECTED,self.REGISTERING,self.REGISTERED)
        def onResult( err, result) :
            del self.pingOps[pingOp]
            if err < 0 :
                pingCallback( None )
                return
            assert type(result) is list
            if (len(result) != 1) or (type(result[0]) is not list) :
                pingCallback( None )
                return
            addr = tuple(result[0])
            try :
                validateInetAddress( addr )
            except :
                pingCallback( None )
                return
            pingCallback( addr )
        pingOp = _doRPCCall( self.rpcConn, 'Ping', [], onResult )
        self.pingOps[pingOp] = 1

    def callRegister( self, registerCallback, incomingCallback ) :
        assert self.state == self.CONNECTED
        def onResult( err, result ) :
            self.registerOp = None
            if err < 0 :
                self.state = self.CONNECTED
                registerCallback( None )
                return
            if (len(result) != 1) or \
                    (type(result[0]) is not str) or (not result[0]) :
                self.state = self.CONNECTED
                registerCallback( None )
                return
            self.state = self.REGISTERED
            self.routerId = result[0]
            self.rpcConn.setOnewayCallback( self._onOneway )
            self.incomingCallback = incomingCallback
            registerCallback( self.routerId )
        self.registerOp = _doRPCCall( self.rpcConn, 'Register', [], onResult )
        self.state = self.REGISTERING

    def _onOneway( self, payload ) :
        assert self.state == self.REGISTERED
        global logger
        if (type(payload) is not str) or (not payload) :
            return
        # logger.debug( 'oneway payload ='+str(payload) )
        connectionId = payload
        self.incomingCallback( connectionId )



import logging
from time import time
from socket import error as sock_error
from nitro.errors import EWOULDBLOCK, EAGAIN

logger = logging.getLogger('cspace.nitro.tcpbridge')

class StreamState(object) :
    def __init__( self, srcSock, destSock ) :
        self.src = srcSock
        self.dest = destSock
        self.srcClosed = False
        self.destClosed = False
        self.data = ''
        self.maxData = 65536
        self.hasRead = False
        self.hasWrite = False

    def canRead( self ) :
        if self.destClosed or self.srcClosed : return False
        if len(self.data) == self.maxData : return False
        return True

    def canWrite( self ) :
        if self.destClosed : return False
        if len(self.data) == 0 : return False
        return True

class TCPBridge(object) :
    def __init__( self, sock1, sock2, reactor ) :
        global logger
        self.socks = [ sock1, sock2 ]
        self.id = '%s-%s' % (str(sock1.getpeername()), str(sock2.getpeername()))
        logger.info('new bridge %s' % self.id)
        self.reactor = reactor
        self.streams = [ StreamState(sock1,sock2), StreamState(sock2,sock1) ]
        for s in self.streams : self.updateEvents(s)
        for i in range(2) : self.streams[i].index = i
        self.closeCallback = None
        self._setActive()
        
    def setCloseCallback( self, closeCallback ) :
        self.closeCallback = closeCallback

    def close( self ) :
        logger.info('close bridge %s' % self.id)
        for stream in self.streams :
            if stream.hasRead :
                self.reactor.removeReadCallback( stream.src.fileno() )
            if stream.hasWrite :
                self.reactor.removeWriteCallback( stream.dest.fileno() )
        for s in self.socks : s.close()

    def lastActiveTime( self ) :
        return self._lastActiveTime

    def _setActive( self ) :
        self._lastActiveTime = time()

    def closeInternal( self ) :
        logger.info('bridge %s closing internal' % self.id)
        for stream in self.streams :
            assert not stream.hasRead
            assert not stream.hasWrite
        for s in self.socks : s.close()
        if self.closeCallback : self.closeCallback()

    def updateEvents( self, stream ) :
        if (stream.srcClosed) and (len(stream.data) == 0) and (not stream.destClosed) :
            try :
                stream.dest.shutdown( 1 )
                stream.destClosed = True
            except sock_error, (err,errMsg) :
                self.handleError( stream.dest )

        readEvent = stream.canRead()
        if readEvent != stream.hasRead :
            stream.hasRead = readEvent
            if readEvent :
                self.reactor.addReadCallback( stream.src.fileno(), lambda : self._onRead(stream) )
            else :
                self.reactor.removeReadCallback( stream.src.fileno() )
        writeEvent = stream.canWrite()
        if writeEvent != stream.hasWrite :
            stream.hasWrite = writeEvent
            if writeEvent :
                self.reactor.addWriteCallback( stream.dest.fileno(), lambda : self._onWrite(stream) )
            else :
                self.reactor.removeWriteCallback( stream.dest.fileno() )

    def handleError( self, sock ) :
        if sock is self.socks[0] :
            self.streams[0].srcClosed = True
            self.streams[1].destClosed = True
        else :
            assert sock is self.socks[1]
            self.streams[0].destClosed = True
            self.streams[1].srcClosed = True

    def _onRead( self, stream ) :
        self._setActive()
        if stream.canRead() :
            try :
                data = stream.src.recv( stream.maxData - len(stream.data) )
                if not data :
                    stream.srcClosed = True
                else :
                    stream.data += data
            except sock_error, (err,errMsg) :
                if err not in (EWOULDBLOCK,EAGAIN) :
                    self.handleError( stream.src )
        if stream.canWrite() :
            try :
                numWritten = stream.dest.send( stream.data )
                stream.data = stream.data[numWritten:]
            except sock_error, (err,errMsg) :
                if err not in (EWOULDBLOCK,EAGAIN) :
                    self.handleError( stream.dest )
        for s in self.streams : self.updateEvents(s)

        if self.streams[0].destClosed and self.streams[1].destClosed :
            self.closeInternal()

    def _onWrite( self, stream ) :
        self._setActive()
        finished = False
        writeBlocked = False
        readBlocked = False
        while stream.canWrite() and not writeBlocked :
            try :
                numWritten = stream.dest.send( stream.data )
                stream.data = stream.data[numWritten:]
            except sock_error, (err,errMsg) :
                if err not in (EWOULDBLOCK,EAGAIN) :
                    self.handleError( stream.dest )
                else :
                    writeBlocked = True

            if stream.canRead() and not readBlocked :
                try :
                    data = stream.src.recv( stream.maxData - len(stream.data) )
                    if not data :
                        stream.srcClosed = True
                    else :
                        stream.data += data
                except sock_error, (err,errMsg) :
                    if err not in (EWOULDBLOCK,EAGAIN) :
                        self.handleError( stream.src )
                    else :
                        readBlocked = True
        for s in self.streams : self.updateEvents(s)

        if self.streams[0].destClosed and self.streams[1].destClosed :
            self.closeInternal()



class StateMachine( object ) :
    def __init__( self, initial ) :
        assert initial >= 0
        self.__current = initial
        self.__previous = None
        self.__nextId = 0
        self.__callbacks = []
        self.__callbackIds = {}
        self.__changing = False

    def current( self ) : return self.__current
    def previous( self ) : return self.__previous

    def insertCallback( self, callback, src=None, dest=None, single=False ) :
        callbackId = self.__nextId
        self.__nextId += 1
        self.__callbacks.insert( 0, (src,dest,callback,callbackId,single) )
        self.__callbackIds[callbackId] = 1
        return callbackId

    def appendCallback( self, callback, src=None, dest=None, single=False ) :
        callbackId = self.__nextId
        self.__nextId += 1
        self.__callbacks.append( (src,dest,callback,callbackId,single) )
        self.__callbackIds[callbackId] = 1
        return callbackId

    def removeCallback( self, callbackId ) :
        del self.__callbackIds[callbackId]
        for i,info in enumerate(self.__callbacks) :
            if info[3] == callbackId :
                del self.__callbacks[i]
                return
        assert False

    def change( self, state ) :
        assert not self.__changing
        if self.__current == state : return
        self.__previous = self.__current
        self.__current = state
        receivers = []
        for src,dest,cb,cbid,single in self.__callbacks :
            if (src is not None) and (src != self.__previous) : continue
            if (dest is not None) and (dest != self.__current) : continue
            receivers.append( (cb,cbid,single) )
        if not receivers : return
        last = receivers.pop()
        self.__changing = True
        for cb,cbid,single in receivers :
            if cbid in self.__callbackIds :
                if single :
                    self.removeCallback( cbid )
                cb()
        self.__changing = False
        cb,cbid,single = last
        if cbid in self.__callbackIds :
            if single :
                self.removeCallback( cbid )
            cb()



from cspace.util.settings import LocalSettings, AppSettings
import logging
logger = logging.getLogger('cspace.common')
SETTINGS_VERSION = 3

_localSettings = None
def localSettings(location='CSpace') :
    global _localSettings
    if _localSettings is None :
        _localSettings = LocalSettings(location)
        logger.info('setup local settings in ' + _localSettings.configDir)
    if not _localSettings.configDir.rstrip('//\/').endswith(location):
        return LocalSettings(location)
    return _localSettings

_profileSettings = None
def profileSettings(location='CSpaceProfiles'):
    global _profileSettings
    if _profileSettings is None :
        _profileSettings = LocalSettings(location)
        logger.info('setup profile settings in ' + _profileSettings.configDir)
    if not _profileSettings.configDir.rstrip('//\/').endswith(location):
        return LocalSettings(location)
    return _profileSettings

_appSettings = None
def appSettings() :
    global _appSettings
    if _appSettings is None :
        _appSettings = AppSettings()
        logger.info('setup app settings in ' + _appSettings.configDir)
    return _appSettings

MAX_NAME_LENGTH = 32

def _init() :
    global validUserChars, validServiceChars
    validUserChars = set("abcdefghijklmnopqrstuvwxyz0123456789-_")
    validServiceChars = set("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_")
_init()

def isValidUserName( name ) :
    if not (0 < len(name) <= MAX_NAME_LENGTH) : return False
    for x in name :
        if not x in validUserChars : return False
    return True

def isValidServiceName( name ) :
    if not (0 < len(name) <= MAX_NAME_LENGTH) : return False
    for x in name :
        if not x in validServiceChars : return False
    return True



#!/usr/bin/env python
#central_connector.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in central_connector.py')
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet.task import LoopingCall


import lib.dhnio as dhnio
import lib.misc as misc
import lib.packetid as packetid
import lib.settings as settings
import lib.contacts as contacts
import lib.transport_control as transport_control
if transport_control._TransportCSpaceEnable:
    import lib.transport_cspace as transport_cspace
from lib.automat import Automat


import lib.automats as automats
import network_connector
import p2p_connector
import central_service

import dhnicon
import money
import identitypropagate


_CentralConnector = None

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _CentralConnector
    if _CentralConnector is None:
        _CentralConnector = CentralConnector('central_connector', 'AT_STARTUP', 4)
    if event is not None:
        _CentralConnector.automat(event, arg)
    return _CentralConnector

class CentralConnector(Automat):
    timers = {'timer-1hour':  (60*60, ['CONNECTED', 'DISCONNECTED', 'ONLY_ID']),
              'timer-30sec':  (30,    ['IDENTITY', 'REQUEST_SETTINGS', 'SETTINGS', 'SUPPLIERS', 'DISCONNECTED']),
              'timer-10sec':  (10,    ['IDENTITY'])}
    flagSettings = False

    def state_changed(self, oldstate, newstate):
        automats.set_global_state('CENTRAL ' + newstate)
        p2p_connector.A('central_connector.state', newstate)
        dhnicon.state_changed(network_connector.A().state, p2p_connector.A().state, self.state)

    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state is 'AT_STARTUP':
            if event == 'propagate' :
                self.state = 'IDENTITY'
                self.flagSettings=False
                self.doInitCentralService(arg)
                self.doSendIdentity(arg)
        #---IDENTITY---
        elif self.state is 'IDENTITY':
            if event == 'identity-ack' and not self.isSettingsExist(arg) :
                self.state = 'REQUEST_SETTINGS'
                self.doSendRequestSettings(arg)
            elif event == 'identity-ack' and self.isSettingsExist(arg) and not self.flagSettings :
                self.state = 'SETTINGS'
                self.doSendSettings(arg)
            elif event == 'identity-ack' and self.isSettingsExist(arg) and self.flagSettings and not self.isSuppliersNeeded(arg) :
                self.state = 'CONNECTED'
                self.doSendRemainingPackets(arg)
            elif event == 'identity-ack' and self.flagSettings and self.isSettingsExist(arg) and self.isSuppliersNeeded(arg) :
                self.state = 'SUPPLIERS'
                self.doSendRequestSuppliers(arg)
            elif event == 'timer-10sec' :
                self.doSendIdentity(arg)
            elif event == 'timer-30sec' :
                self.state = 'DISCONNECTED'
        #---SETTINGS---
        elif self.state is 'SETTINGS':
            if event == 'settings-ack' :
                self.state = 'SUPPLIERS'
                self.flagSettings=True
            elif event == 'list-suppliers' :
                self.state = 'CONNECTED'
                self.flagSettings=True
                self.doSendRemainingPackets(arg)
            elif event == 'propagate' :
                self.state = 'IDENTITY'
                self.doSendIdentity(arg)
            elif event == 'timer-30sec' :
                self.state = 'ONLY_ID'
            elif event == 'settings' :
                self.doSendSettings(arg)
        #---REQUEST_SETTINGS---
        elif self.state is 'REQUEST_SETTINGS':
            if event == 'request-settings-ack' and self.isSuppliersNeeded(arg) :
                self.state = 'SUPPLIERS'
                self.flagSettings=True
                self.doSendRequestSuppliers(arg)
            elif event == 'request-settings-ack' and not self.isSuppliersNeeded(arg) :
                self.state = 'CONNECTED'
                self.flagSettings=True
                self.doSendRemainingPackets(arg)
            elif event == 'propagate' :
                self.state = 'IDENTITY'
                self.doSendIdentity(arg)
            elif event == 'timer-30sec' :
                self.state = 'ONLY_ID'
        #---SUPPLIERS---
        elif self.state is 'SUPPLIERS':
            if event == 'list-suppliers' or event == 'list-customers' :
                self.state = 'CONNECTED'
                self.doSendRemainingPackets(arg)
            elif event == 'propagate' :
                self.state = 'IDENTITY'
                self.doSendIdentity(arg)
            elif event == 'timer-30sec' :
                self.state = 'ONLY_ID'
        #---ONLY_ID---
        elif self.state is 'ONLY_ID':
            if event == 'propagate' or event == 'list-suppliers' or event == 'timer-1hour' or event == 'identity-ack' or event == 'list-customers' or event == 'settings' :
                self.state = 'IDENTITY'
                self.flagSettings=False
                self.doSendIdentity(arg)
        #---CONNECTED---
        elif self.state is 'CONNECTED':
            if event == 'settings' :
                self.state = 'SETTINGS'
                self.doSendSettings(arg)
                self.flagSettings=False
            elif event == 'propagate' or event == 'timer-1hour' :
                self.state = 'IDENTITY'
                self.doSendIdentity(arg)
        #---DISCONNECTED---
        elif self.state is 'DISCONNECTED':
            if event == 'propagate' or event == 'list-suppliers' or event == 'timer-1hour' or event == 'identity-ack' or event == 'list-customers' or event == 'settings' or ( event == 'p2p_connector.state' and arg is 'CONNECTED' ) or ( event == 'timer-30sec' and self.isCSpaceOnline(arg) ) :
                self.state = 'IDENTITY'
                self.flagSettings=False
                self.doSendIdentity(arg)

    def isSettingsExist(self, arg):
        return settings.getCentralNumSuppliers() > 0

    def isSuppliersNeeded(self, arg):
        return settings.getCentralNumSuppliers() <= 0 or \
               contacts.numSuppliers() != settings.getCentralNumSuppliers()

    def _saveRequestedSettings(self, newpacket):
        sd = dhnio._unpack_dict(newpacket.Payload)
        settings.uconfig().set('central-settings.needed-megabytes', sd.get('n', str(settings.DefaultNeededMb()))+'MB')
        settings.uconfig().set('central-settings.shared-megabytes', sd.get('d', str(settings.DefaultDonatedMb()))+'MB')
        settings.uconfig().set('central-settings.desired-suppliers', sd.get('s', '2'))
        settings.uconfig().set('emergency.emergency-email', sd.get('e1', ''))
        settings.uconfig().set('emergency.emergency-phone', sd.get('e2', ''))
        settings.uconfig().set('emergency.emergency-fax', sd.get('e3', ''))
        settings.uconfig().set('emergency.emergency-text', sd.get('e4', '').replace('<br>', '\n'))
        settings.uconfig().update()
        reactor.callLater(0, self.automat, 'request-settings-ack', newpacket)

    def doInitCentralService(self, arg):
        central_service.init()

    def doSendIdentity(self, arg):
        transport_control.RegisterInterest(
            lambda packet: self.automat('identity-ack', packet),
            settings.CentralID(),
            central_service.SendIdentity(True))

    def doSendSettings(self, arg):
        packetID = packetid.UniqueID()
        transport_control.RegisterInterest(
            self.settingsAck,
            settings.CentralID(),
            packetID)
        central_service.SendSettings(True, packetID)

    def doSendRequestSettings(self, arg):
        transport_control.RegisterInterest(
            lambda packet: self._saveRequestedSettings(packet),
            settings.CentralID(),
            central_service.SendRequestSettings(True))

    def doSendRequestSuppliers(self, arg):
        return central_service.SendRequestSuppliers()

    def isCSpaceOnline(self, arg):
        if not transport_control._TransportCSpaceEnable:
            return False
        if not settings.enableCSpace():
            return False
        if not transport_cspace.registered():
            return False
        return transport_cspace.A().state == 'ONLINE'

    def settingsAck(self, packet):
        try:
            status, last_receipt, = packet.Payload.split('\n', 2)
            last_receipt = int(last_receipt)
        except:
            status = 'error'
            last_receipt = -1
        dhnio.Dprint(4, 'central_connector.settingsAck [%s] last_receipt=%d' % (status, last_receipt))
        missing_receipts = money.SearchMissingReceipts(last_receipt)
        if len(missing_receipts) > 0:
            reactor.callLater(0, central_service.SendRequestReceipt, missing_receipts)
        self.automat('settings-ack', packet)

    def doSendRemainingPackets(self, arg):
        central_service.LoopSendBandwidthReport()
        central_service.SendRequestMarketList()
        
        



import sys, ctypes
try:
    from PyQt4 import QtGui
except:
    pass 
from nitro.async import AsyncOp

if sys.platform == 'win32' :
    def _flash( window, yes ) :
        ctypes.windll.user32.FlashWindow( int(window.winId()), yes )

    def _isForegroundWindow( window ) :
        return int(window.winId()) == ctypes.windll.user32.GetForegroundWindow()

    def _startWindowFlashing( window, reactor, callback=None ) :
        _flash( window, False )
        _flash( window, True )
        def onTimer() :
            if _isForegroundWindow(window) :
                doCancel()
                op.notify()
                return
            _flash( window, True )
        def doCancel() :
            timerOp.cancel()
            _flash( window, False )
        timerOp = reactor.addTimer( 1, onTimer )
        op = AsyncOp( callback, doCancel )
        return op 

    class FlashWindow( object ) :
        def __init__( self, reactor ) :
            self.__flashOp = None
            self.__reactor = reactor

        def flash( self ) :
            if self.__flashOp : return
            if _isForegroundWindow(self) : return
            def onFlashComplete() :
                self.__flashOp = None
            self.__flashOp = _startWindowFlashing( self,
                    self.__reactor, onFlashComplete )

        def cancelFlash( self ) :
            if self.__flashOp is not None :
                self.__flashOp.cancel()
     
elif sys.platform.startswith('linux') :
    class _XClientMessage_data( ctypes.Union ) :
        _fields_ = [('b', ctypes.c_char*20),
                    ('s', ctypes.c_short*10),
                    ('l', ctypes.c_long*5)
                ]

    class XClientMessageEvent( ctypes.Structure ) :
        _fields_ = [('type', ctypes.c_int),
                    ('serial', ctypes.c_ulong),
                    ('send_event', ctypes.c_int),
                    ('display', ctypes.c_void_p),
                    ('window', ctypes.c_ulong),
                    ('message_type', ctypes.c_ulong),
                    ('format', ctypes.c_int),
                    ('data', _XClientMessage_data)
                ]

    class XEvent( ctypes.Union ) :
        _fields_ = [('xclient', XClientMessageEvent),
                    ('padding', ctypes.c_char*96)
                ]

    ClientMessage = 33
    SubstructureRedirectmask = 1048576
    SubstructureNotifyMask = 524288

    def _isForegroundWindow( window ) :
        aWin = QtGui.QApplication.activeWindow()
        if aWin is None :
            return False
        else :
            return int(window.winId()) == aWin.winId()

    def _flash( window, yes ) :
        xdisplay = ctypes.c_void_p( int(QtGui.QX11Info.display()) )
        rootwin = QtGui.QX11Info.appRootWindow()
        winId = int(window.winId())
        try:
            X11 = ctypes.cdll.X11
        except:
            pass
        else:
            demandsAttention = X11.XInternAtom( xdisplay, "_NET_WM_STATE_DEMANDS_ATTENTION", 1 )
            wmState = X11.XInternAtom( xdisplay, "_NET_WM_STATE", 1 )
            e = XEvent()
            e.xclient.type = ClientMessage
            e.xclient.message_type = wmState
            e.xclient.display = xdisplay
            e.xclient.window = winId
            e.xclient.format = 32
            e.xclient.data.l[1] = demandsAttention
            e.xclient.data.l[2] = 0
            e.xclient.data.l[3] = 0
            e.xclient.data.l[4] = 0
    
            if yes :
                e.xclient.data.l[0] = 1
            else :
                e.xclient.data.l[0] = 0
            X11.XSendEvent( xdisplay, rootwin, 0, (SubstructureRedirectmask |
                                                   SubstructureNotifyMask),
                            ctypes.pointer(e) )

    class FlashWindow( object ) :
        def __init__( self, reactor ) :
            self.__reactor = reactor
            self.__flashing = False
            self.__timer = None

        def __stopFlash( self ) :
            _flash( self, False )
            self.__flashing = False
            self.__timer.cancel()
            self.__timer = None

        def flash( self ) :
            if self.__flashing :
                return
            _flash( self, True )
            self.__flashing = True
            def onTimer( ) :
                if _isForegroundWindow( self ) :
                    self.__stopFlash()
            self.__timer = self.__reactor.addTimer( 1, onTimer )

        def cancelFlash( self ) :
            if self.__flashing :
                self.__stopFlash()

else :
    class FlashWindow( object ) :
        def __init__( self, reactor ) :
            pass

        def flash( self ) :
            pass

        def cancelFlash( self ) :
            pass



#!/usr/bin/python
#backup_rebuilder.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time
import random


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backup_rebuilder.py')

from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet import threads


import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.settings as settings
import lib.contacts as contacts
import lib.eccmap as eccmap
import lib.tmpfile as tmpfile
import lib.diskspace as diskspace
from lib.automat import Automat


import fire_hire
import backup_monitor
import block_rebuilder
import lib.automats as automats

import backups
import raidread
import io_throttle
import backup_db

_BackupRebuilder = None
_StoppedFlag = True
_BackupIDsQueue = []  
_BlockRebuildersQueue = []

#------------------------------------------------------------------------------ 

def A(event=None, arg=None):
    global _BackupRebuilder
    if _BackupRebuilder is None:
        _BackupRebuilder = BackupRebuilder('backup_rebuilder', 'STOPPED', 12)
    if event is not None:
        _BackupRebuilder.automat(event, arg)
    return _BackupRebuilder


class BackupRebuilder(Automat):
    timers = {'timer-1sec':     (1,     ['NEXT_BLOCK', 'NEXT_BACKUP']),
              'timer-01sec':    (0.1,   ['NEXT_BLOCK']), }
    
    def init(self):
        self.currentBackupID = None             # currently working on this backup
        self.currentBlockNumber = -1            # currently working on this block
        self.workingBlocksQueue = []            # list of missing blocks we work on for current backup 
        self.maxBlockRebuilderQueueLength = 8   # TODO - need to sort out what is an appropriate length

    def state_changed(self, oldstate, newstate):
        # automats.set_global_state('REBUILD ' + newstate)
        backup_monitor.A('backup_rebuilder.state', newstate)
        
    def A(self, event, arg):
        #---STOPPED---
        if self.state is 'STOPPED':
            if event == 'start' :
                self.state = 'NEXT_BACKUP'
                self.doClearStoppedFlag(arg)
            elif event == 'init' :
                pass
        #---NEXT_BACKUP---
        elif self.state is 'NEXT_BACKUP':
            if event == 'timer-1sec' and not self.isStopped(arg) and self.isMoreBackups(arg) :
                self.state = 'NEXT_BLOCK'
                self.doTakeNextBackup(arg)
                self.doTakeNextBlock(arg)
            elif event == 'timer-1sec' and not self.isMoreBackups(arg) and not self.isStopped(arg) :
                self.state = 'DONE'
            elif event == 'timer-1sec' and self.isStopped(arg) :
                self.state = 'STOPPED'
        #---NEXT_BLOCK---
        elif self.state is 'NEXT_BLOCK':
            if event == 'timer-01sec' and not self.isStopped(arg) and not self.isMoreBlocks(arg) and self.isMoreBackups(arg) :
                self.state = 'NEXT_BACKUP'
            elif event == 'timer-01sec' and ( ( not self.isMoreBackups(arg) and not self.isMoreBlocks(arg) ) or self.isFailedSuppliers(arg) ) :
                self.state = 'DONE'
            elif event == 'timer-1sec' and self.isStopped(arg) :
                self.state = 'STOPPED'
            elif ( event == 'timer-01sec' or event == 'block-done' ) and not self.isStopped(arg) and self.isMoreBlocks(arg) and not self.isFailedSuppliers(arg) and self.isSpaceForNewBlockRebuilder(arg) :
                self.state = 'REBUILDING'
                self.doStartNewBlockRebuilder(arg)
        #---REBUILDING---
        elif self.state is 'REBUILDING':
            if ( event == 'block_rebuilder.state' and arg in [ 'FINISHED' , 'STOPPED' , 'DONE' ] ) :
                self.state = 'NEXT_BLOCK'
                self.doTakeNextBlock(arg)
            elif ( event == 'block_rebuilder.state' and arg is 'TIMEOUT' ) :
                self.state = 'DONE'
        #---DONE---
        elif self.state is 'DONE':
            if event == 'start' :
                self.state = 'NEXT_BACKUP'
                self.doClearStoppedFlag(arg)

    def isStopped(self, arg):
        return ReadStoppedFlag() == True # :-)

    def isMoreBackups(self, arg):
        global _BackupIDsQueue
        return len(_BackupIDsQueue) > 0
    
    def isMoreBlocks(self, arg):
        # because started from 0,  -1 means not found 
        return self.currentBlockNumber > -1 
    
    def isSpaceForNewBlockRebuilder(self, arg):
        global _BlockRebuildersQueue
        return len(_BlockRebuildersQueue) < self.maxBlockRebuilderQueueLength

    def isFailedSuppliers(self, arg):
        # failed_suppliers = backups.GetFailedSuppliers()
        # dhnio.Dprint(6, 'backup_rebuilder.isFailedSuppliers return %s: %s' % (str(len(failed_suppliers) > 0), failed_suppliers))
        # return len(failed_suppliers) > 0
        return False

    def doTakeNextBackup(self, arg):
        global _BackupIDsQueue
        # take a first backup from queue to work on it
        backupID = _BackupIDsQueue.pop(0)
        # dhnio.Dprint(6, 'backup_rebuilder.doTakeNextBackup first backup in queue is %s' % backupID)
        # if remote data structure is not exist for this backup - create it
        # this mean this is only local backup!
        if not backups.remote_files().has_key(backupID):
            backups.remote_files()[backupID] = {}
            # we create empty remote info for every local block
            # range(0) should return []
            for blockNum in range(backups.local_max_block_numbers().get(backupID, -1) + 1):
                backups.remote_files()[backupID][blockNum] = {
                    'D': [0] * backups.suppliers_set().supplierCount,
                    'P': [0] * backups.suppliers_set().supplierCount }
        # clear blocks queue from previous iteration
        self.currentBlockNumber = -1
        # del self.workingBlocksQueue
        self.workingBlocksQueue = [] 
        # detect missing blocks from remote info
        self.workingBlocksQueue = backups.ScanMissingBlocks(backupID)
        # find the correct max block number for this backup
        # we can have remote and local files
        # will take biggest block number from both 
        backupMaxBlock = max(backups.remote_max_block_numbers().get(backupID, -1),
                             backups.local_max_block_numbers().get(backupID, -1))
        # now need to remember this biggest block number
        # remote info may have less blocks - need to create empty info for missing blocks
        for blockNum in range(backupMaxBlock + 1):
            if backups.remote_files()[backupID].has_key(blockNum):
                continue
            backups.remote_files()[backupID][blockNum] = {
                'D': [0] * backups.suppliers_set().supplierCount,
                'P': [0] * backups.suppliers_set().supplierCount }
        # really take next backup
        self.currentBackupID = backupID
        # clear sending queue, remove old packets for this backup, we will send them again
        io_throttle.DeleteBackupRequests(self.currentBackupID)
        # dhnio.Dprint(6, 'backup_rebuilder.doTakeNextBackup currentBackupID=%s workingBlocksQueue=%d' % (self.currentBackupID, len(self.workingBlocksQueue)))
       
    def doTakeNextBlock(self, arg):
        if len(self.workingBlocksQueue) > 0:
            # let's take last blocks first ... 
            # in such way we can propagate how big is the whole backup as soon as possible!
            # remote machine can multiply [file size] * [block number] 
            # and calculate the whole size to be received ... smart!
            # ... remote supplier should not use last file to calculate
            self.currentBlockNumber = self.workingBlocksQueue.pop()
            # self.currentBlockNumber = self.workingBlocksQueue.pop(0)
        else:
            self.currentBlockNumber = -1
        self.automat('block-done')

    def doStartNewBlockRebuilder(self, arg):
        br = block_rebuilder.BlockRebuilder(
            eccmap.Current(), #self.eccMap,
            self.currentBackupID,
            self.currentBlockNumber,
            backups.suppliers_set(),
            backups.GetRemoteDataArray(self.currentBackupID, self.currentBlockNumber),
            backups.GetRemoteParityArray(self.currentBackupID, self.currentBlockNumber),
            backups.GetLocalDataArray(self.currentBackupID, self.currentBlockNumber),
            backups.GetLocalParityArray(self.currentBackupID, self.currentBlockNumber),)
        AddBlockRebuilder(br)
        br.automat('start')

    def doClearStoppedFlag(self, arg):
        ClearStoppedFlag()


#    def doSetStoppedFlag(self):
#        SetStoppedFlag()
        
#    def doStartRepainting(self):
#        RepaintingProcess(True)
        
#    def doStopRepainting(self):
#        RepaintingProcess(False)

#------------------------------------------------------------------------------ 

def AddBackupsToWork(backupIDs):
    global _BackupIDsQueue 
    _BackupIDsQueue.extend(backupIDs)


def RemoveBackupToWork(backupID):
    global _BackupIDsQueue
    if backupID in _BackupIDsQueue:
        _BackupIDsQueue.remove(backupID)


def SetStoppedFlag():
    global _StoppedFlag
    _StoppedFlag = True
    
    
def ClearStoppedFlag():
    global _StoppedFlag
    _StoppedFlag = False
    

def ReadStoppedFlag():
    global _StoppedFlag
    return _StoppedFlag
    
    
def AddBlockRebuilder(obj):
    global _BlockRebuildersQueue
    dhnio.Dprint(6, 'backup_rebuilder.AddBlockRebuilder for %s-%s' % (obj.backupID, str(obj.blockNum)))
    _BlockRebuildersQueue.append(obj)
    

def RemoveBlockRebuilder(obj):
    global _BlockRebuildersQueue
    dhnio.Dprint(6, 'backup_rebuilder.RemoveBlockRebuilder for %s-%s' % (obj.backupID, str(obj.blockNum)))
    _BlockRebuildersQueue.remove(obj)


    
    

    



#!/usr/bin/python
#transport_cspace.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2013
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys
import time 
import xmlrpclib
import subprocess
import base64
import random
import optparse

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in transport_cspace.py')

from twisted.internet.defer import Deferred, succeed, fail
from twisted.web import resource
from twisted.web import server
from twisted.internet import protocol
from twisted.internet import interfaces
from zope.interface import implements

import dhnio
import misc
import settings
import tmpfile
import nonblocking
import automat

#------------------------------------------------------------------------------ 

_Debug = False
_BaseDir = '.'
_Controller = None
_CSpaceProcess = None
_Listener = None
_OutgoingFilesDict = {}
_OutgoingQueue = []
_SendingDelay = 0.01

_SendStatusFunc = None
_ReceiveStatusFunc = None
_RegisterTransferFunc = None
_UnRegisterTransferFunc = None
_ContactStatusNotifyFunc = None

#------------------------------------------------------------------------------

def init(baseDir=None):
    global _Controller
    global _BaseDir
    global _SendStatusFunc 
    global _ReceiveStatusFunc 
    global _RegisterTransferFunc 
    global _UnRegisterTransferFunc 
    dhnio.Dprint(6, 'transport_cspace.init')
    if _Controller is not None:
        return succeed('already started')
    if baseDir:
        _BaseDir = baseDir 
    _Controller = CSpaceServiceController()
    resultDefer = Deferred()
    A('init', resultDefer)
    return resultDefer


def shutdown():
    global _Controller
    if not _Controller:
        dhnio.Dprint(6, 'transport_cspace.shutdown transport is not started, skip')
        return succeed(0)
    dhnio.Dprint(6, 'transport_cspace.shutdown sending event "shutdown" to the state machine')
    d = Deferred()
    A('shutdown', d)
    return d


def shutdown_final(arg=None):
    global _Controller
    global _Listener
    global _OutgoingFilesDict
    global _OutgoingQueue
    global _TransportCspace
    dhnio.Dprint(6, 'transport_cspace.shutdown_final')
    if _Controller is not None:
        dhnio.Dprint(6, 'transport_cspace.shutdown_final stopping controller')
        _Controller.stop()
        _Controller = None
    if _Listener is not None:
        dhnio.Dprint(6, 'transport_cspace.shutdown_final destroying pseudo listener')
        del _Listener
        _Listener = None
    _OutgoingFilesDict.clear()
    _OutgoingQueue = []
    kill()
    del _TransportCspace
    _TransportCspace = None
    if arg and isinstance(arg, Deferred):
        arg.callback(1)


def run(params=[], process_finished_callback=None):
    global _BaseDir
    global _CSpaceProcess
    if _CSpaceProcess is not None:
        return None
    if dhnio.isFrozen() and dhnio.Windows():
        progpath = os.path.abspath(os.path.join(_BaseDir, 'dhnet.exe'))
        executable = progpath
        cmdargs = [progpath,]
        cmdargs.extend(params)
    else:
        progpath = os.path.abspath(os.path.join(_BaseDir, 'dhnet.py'))
        executable = sys.executable
        cmdargs = [sys.executable, progpath]
        cmdargs.extend(params)
    if not os.path.isfile(executable):
        dhnio.Dprint(1, 'transport_cspace.run ERROR %s not found' % executable)
        return None
    if not os.path.isfile(progpath):
        dhnio.Dprint(1, 'transport_cspace.run ERROR %s not found' % progpath)
        return None
    dhnio.Dprint(6, 'transport_cspace.run execute: "%s"' % (' '.join(cmdargs)))

    if dhnio.Windows():
        from twisted.internet import _dumbwin32proc
        real_CreateProcess = _dumbwin32proc.win32process.CreateProcess
        def fake_createprocess(_appName, _commandLine, _processAttributes,
                            _threadAttributes, _bInheritHandles, creationFlags,
                            _newEnvironment, _currentDirectory, startupinfo):
            import win32con
            flags = win32con.CREATE_NO_WINDOW 
            return real_CreateProcess(_appName, _commandLine,
                            _processAttributes, _threadAttributes,
                            _bInheritHandles, flags, _newEnvironment,
                            _currentDirectory, startupinfo)        
        setattr(_dumbwin32proc.win32process, 'CreateProcess', fake_createprocess)
    try:
        _CSpaceProcess = reactor.spawnProcess(CSpaceServiceProtocol(process_finished_callback), executable, cmdargs, path=_BaseDir)
    except:
        dhnio.Dprint(1, 'transport_cspace.run ERROR executing: %s' % str(cmdargs))
        dhnio.DprintException()
        return None
    
    if dhnio.Windows():
        setattr(_dumbwin32proc.win32process, 'CreateProcess', real_CreateProcess)

    dhnio.Dprint(6, 'transport_cspace.run pid=%d' % _CSpaceProcess.pid)
    return _CSpaceProcess


def kill():
    global _CSpaceProcess
    if _CSpaceProcess is not None:
        try:
            _CSpaceProcess.signalProcess('KILL')
            # _CSpaceProcess.kill()
            dhnio.Dprint(6, 'transport_cspace.kill finished "dhnet" process')
        except:
            pass
    del _CSpaceProcess
    _CSpaceProcess = None
    killed = False
    for pid in dhnio.find_process(['dhnet.']): 
        dhnio.kill_process(pid)
        dhnio.Dprint(6, 'transport_cspace.kill pid %d' % pid)
        killed = True
    if killed:
        pidpath = getPIDpath()
        if os.path.isfile(pidpath):
            try:
                os.remove(pidpath)
                dhnio.Dprint(6, 'transport_cspace.kill %s removed' % pidpath)
            except:
                pass

    
def alive():
    global _CSpaceProcess
    if _CSpaceProcess is None:
        return False
    try:
        pid = _CSpaceProcess.pid
    except:
        return False
    return pid > 0


def registered():    
    exists = [ os.path.isfile(settings.CSpaceRememberKeyFile()),
               os.path.isfile(settings.CSpaceSavedProfileFile()),
               os.path.isfile(settings.CSpaceSavedPasswordFile()),]
    if False in exists:
        return False
    cur_profile_name = dhnio.ReadTextFile(settings.CSpaceSavedProfileFile())
    cur_profile_dir = os.path.join(settings.CSpaceProfilesDir(), cur_profile_name.strip('"'))
    if not os.path.isdir(cur_profile_dir):
        return False
    exists = [os.path.isfile(os.path.join(cur_profile_dir, 'PrivateKey')),
              os.path.isfile(os.path.join(cur_profile_dir, 'KeyID')),]
    if False in exists:
        return False
    return True


def keyID():
    if not registered():
        return None
    cur_profile_name = dhnio.ReadTextFile(settings.CSpaceSavedProfileFile())
    cur_profile_dir = os.path.join(settings.CSpaceProfilesDir(), cur_profile_name.strip('"'))
    return dhnio.ReadTextFile(os.path.join(cur_profile_dir, 'KeyID'))
    

def control():
    global _Controller
    if _Controller is None:
        init()
    return _Controller


def zero():
    return 0


def getfilesize(sz):
    def _sz():
        return sz
    return _sz


def send(filename, keyid, description=''):
    if not os.path.isfile(filename):
        sendingReport(keyid, filename, 'failed', None, 'file not exist')
        return 
    if not os.access(filename, os.R_OK):
        sendingReport(keyid, filename, 'failed', None, 'file can not be read')
        return 
    if A().state is not 'ONLINE':
        sendingReport(keyid, filename, 'failed', None, 'current state is %s' % A().state)
        return
    try:
        size = os.path.getsize(filename)
    except:
        # dhnio.DprintException()
        sendingReport(keyid, filename, 'failed', None, 'file read error')
        return
    transferID = registerTransfer('send', keyid, filename, size, getfilesize(size), description)
    control().send(keyid, filename, transferID, description)

def cancel(transferID):
    control().cancel(transferID)

#------------------------------------------------------------------------------ 

def SetOpenedReceivingStreamsNumber(num):
    control().set_receiving_streams(num)
    
def SetOpenedSendingStreamsNumber(num):
    control().set_sending_streams(num)    

#------------------------------------------------------------------------------ 

def sendingReport(keyid, filename, status, error=None, message=''):
    global _SendStatusFunc
    if _SendStatusFunc:
        return _SendStatusFunc(keyid, filename, status, 'cspace', error, message)
    dhnio.Dprint(6, 'transport_cspace.sendingReport %s to %s is %s, %s' % (filename, keyid, status, message))


def receivingReport(keyID, filename, status, error=None, message=''):
    global _ReceiveStatusFunc
    if _ReceiveStatusFunc:
        return _ReceiveStatusFunc(filename, status, 'cspace', keyID, error, message)
    dhnio.Dprint(6, 'transport_cspace.receivingReport %s from %s is %s' % (filename, keyID, status))


def registerTransfer(send_or_receive, keyid, filename, size, callback, description):
    global _RegisterTransferFunc
    if _RegisterTransferFunc:
        return _RegisterTransferFunc(send_or_receive, keyid, callback, filename, size, description)
    return 1


def unregisterTransfer(transfer_id):
    global _UnRegisterTransferFunc
    if _UnRegisterTransferFunc:
        return _UnRegisterTransferFunc(transfer_id)
    return True


def callback(command, data):
    global _ContactStatusNotifyFunc
    # dhnio.Dprint(8, 'transport_cspace.callback %s %s' % (command, str(data)))
    if command == 'sent':
        try:
            (keyid, filename, transferID, status, err, msg) = data
            if transferID:
                transferID = int(transferID)
            msg = msg.strip('"')
        except:
            dhnio.Dprint(2, 'transport_cspace.callback ERROR data=%s' % str(data))
            dhnio.DprintException()
            return
        sendingReport(keyid, filename, status, err, msg)
        if transferID:
            unregisterTransfer(transferID)
        else:
            dhnio.Dprint(2, 'transport_cspace.callback ERROR transferID is None: %s' % str(data))
            
    elif command == 'receive':
        try:
            (keyid, filename, status, err, msg) = data
            msg = msg.strip('"')
        except:
            dhnio.Dprint(2, 'transport_cspace.callback ERROR data=%s' % str(data))
            dhnio.DprintException()
            return
        try:
            size = os.path.getsize(filename)
        except:
            dhnio.DprintException()
            size = 1
        transferID = registerTransfer('receive', keyid, filename, size, getfilesize(size), '')
        receivingReport(keyid, filename, status, err, msg)
        unregisterTransfer(transferID)
        
    elif command == 'contact':
        if _ContactStatusNotifyFunc:
            try:
                contact, status = data
            except:
                dhnio.Dprint(2, 'transport_cspace.callback ERROR data=%s' % str(data))
                dhnio.DprintException()
                return
            _ContactStatusNotifyFunc(contact, status)
    else:
        if _Controller:
            A(command, data) 


def getPIDpath(appName="CSpace"):
    pidfile = "%s.run" % appName
    pidpath = settings.CSpaceDir()
    if not os.path.exists(pidpath):
        os.makedirs(pidpath)
    pidpath = os.path.join(pidpath, pidfile)
    return pidpath


def pushFileName(SendToKeyID, filename):
    global _OutgoingQueue
    global _OutgoingFilesDict
    if not _OutgoingFilesDict.has_key(SendToKeyID):
        _OutgoingFilesDict[SendToKeyID] = list()
    _OutgoingFilesDict[SendToKeyID].append(filename)
    _OutgoingQueue.append(SendToKeyID)


def popFileName(SendToKeyID):
    global _OutgoingFilesDict
    if not _OutgoingFilesDict.has_key(SendToKeyID):
        return ''
    try:
        return _OutgoingFilesDict[SendToKeyID].pop(0)
    except:
        return ''


def readSendingQueue():
    global _OutgoingQueue
    if len(_OutgoingQueue) == 0:
        return None
    return _OutgoingQueue.pop(0)


def setBaseDir(baseDir):
    global _BaseDir
    _BaseDir = baseDir    

#------------------------------------------------------------------------------ 

_TransportCspace = None
def A(event=None, arg=None):
    global _TransportCspace
    if event == 'shutdown' and _TransportCspace is None:
        shutdown_final(arg)
        return None
    if _TransportCspace is None:
        # set automat name and starting state here
        _TransportCspace = TransportCspace('transport_cspace', 'AT_STARTUP', 6)
    if event is not None:
        _TransportCspace.automat(event, arg)
    return _TransportCspace

# DataHaven.NET transport_cspace() Automat
class TransportCspace(automat.Automat):
    timers = { 'timer-1min': (60, ['OFFLINE']), }
    resultDefer = None
    shutdownDefer = None
    def state_changed(self, oldstate, newstate):
        if newstate in ['ONLINE', 'OFFLINE'] and self.resultDefer is not None and not self.resultDefer.called:
            self.resultDefer.callback(newstate.lower())
            self.resultDefer = None
    def A(self, event, arg):
        #---AT_STARTUP---
        if self.state is 'AT_STARTUP':
            if event == 'init' and not self.isRegistered(arg) :
                self.state = 'REGISTER'
                self.doRegister(arg)
            elif event == 'init' and self.isRegistered(arg) :
                self.state = 'SERVICE'
                self.doExecute(arg)
        #---SERVICE---
        elif self.state is 'SERVICE':
            if event == 'service-started' :
                self.state = 'CONNECTION'
                self.doConnect(arg)
            elif event == 'execute-error' or event == 'shutdown' :
                self.state = 'SHUTDOWN'
                self.doShutdown(arg)
        #---CONNECTION---
        elif self.state is 'CONNECTION':
            if event == 'online' :
                self.state = 'ONLINE'
                self.doProcessSending(arg)
            elif event == 'offline' :
                self.state = 'OFFLINE'
            elif event == 'connect-error' or event == 'shutdown' :
                self.state = 'SHUTDOWN'
                self.doShutdown(arg)
        #---ONLINE---
        elif self.state is 'ONLINE':
            if event == 'offline' :
                self.state = 'OFFLINE'
            elif event == 'shutdown' :
                self.state = 'DISCONNECTION'
                self.doDisconnect(arg)
        #---OFFLINE---
        elif self.state is 'OFFLINE':
            if event == 'shutdown' :
                self.state = 'SHUTDOWN'
                self.doShutdown(arg)
            elif event == 'timer-1min' :
                self.state = 'CONNECTION'
                self.doConnect(arg)
            elif event == 'online' :
                self.state = 'ONLINE'
                self.doProcessSending(arg)
        #---REGISTER---
        elif self.state is 'REGISTER':
            if event == 'register-failed' :
                self.state = 'SHUTDOWN'
                self.doShutdown(arg)
            elif event == 'registered' :
                self.state = 'SERVICE'
                self.doExecute(arg)
        #---SHUTDOWN---
        elif self.state is 'SHUTDOWN':
            pass
        #---DISCONNECTION---
        elif self.state is 'DISCONNECTION':
            if event == 'offline' or event == 'timer-1min' :
                self.state = 'SHUTDOWN'
                self.doShutdown(arg)

    def isRegistered(self, arg):
        return registered()

    def doExecute(self, arg):
        global _Debug
        dhnio.Dprint(6, 'transport_cspace.doExecute')
        if arg and self.resultDefer is None:
            self.resultDefer = arg
        kill()
        args = ['--logfile', '"%s"' % settings.CSpaceLogFilename(), 
                '--homedir', '"%s"' % settings.CSpaceDir(),
                '--tmpdir', '"%s"' % os.path.join(settings.TempDir(), 'cspace-in'),]
        if _Debug:
            args.insert(0, '--debug')
        if run(params=args) is None:
            self.event('execute-error')
            self.resultDefer.callback('execute-error')

    def doShutdown(self, arg):
        automat.objects().pop(self.index)
        d = self.shutdownDefer
        if d is None:
            d = arg
        shutdown_final(d)

    def doConnect(self, arg):
        dhnio.Dprint(6, 'transport_cspace.doConnect keyID=%s' % keyID())
        control().online()

    def doDisconnect(self, arg):
        dhnio.Dprint(6, 'transport_cspace.doDisconnect arg=%s' % str(arg))
        self.shutdownDefer = arg
        control().offline()

    def doProcessSending(self, arg):
        dhnio.Dprint(6, 'transport_cspace.doProcessSending')

    def doRegister(self, arg):
        global _Debug
        dhnio.Dprint(6, 'transport_cspace.doRegister')
        self.resultDefer = arg
        kill()
        username = dhnio.ReadTextFile(settings.UserNameFilename()).strip() 
        if not username:
            username = 'dhn' + time.strftime('%Y%m%d%H%M%S')
        password = base64.b64encode(os.urandom(12))
        settings.setCSpaceUserName(username)
        settings.setCSpacePassword(password)
        def _process_finished(result):
            if not registered():
                self.event('register-failed')
                self.resultDefer.callback('register-failed')
                return
            self.event('registered')
        def _register(username, password):
            args = [ '--register', username, password,
                     '--logfile', '"%s"' % settings.CSpaceLogFilename(),  
                     '--homedir', '"%s"' % settings.CSpaceDir(),
                     '--tmpdir', '"%s"' % os.path.join(settings.TempDir(), 'cspace-in'), ]
            if _Debug:
                args.insert(0, '--debug')
            if run(process_finished_callback=_process_finished, params=args) is None:
                self.event('register-failed')
                self.resultDefer.callback('register-failed')
                return
        reactor.callLater(0, _register, username, password)


#------------------------------------------------------------------------------ 

class CSpaceServiceController():
    def __init__(self):
        self._netserver = None
        self._server = None

    def _getserviceinfo(self):
        pidpath = getPIDpath()
        if not os.path.exists(pidpath) or not os.path.isfile(pidpath):
            return None, None, None
        fInfo = open(pidpath)
        pid = int(fInfo.readline())
        port = int(fInfo.readline())
        applet = int(fInfo.readline())
        return pid, port, applet

    def _getserver(self, netserver=False):
        if self._server is None:
            pid, port, applet = self._getserviceinfo()
            if pid is None:
                return None
            url = "http://localhost:%i/" % port
            dhnio.Dprint(6, 'transport_cspace._getserver connecting to XMLRPC server on %s' % url)
            self._server = xmlrpclib.Server(url, allow_none=True,)
        return self._server
    
    def online(self):
        server = self._getserver()
        if server is not None:
            try:
                result = server.online()
            except:
                dhnio.DprintException()
                return
            dhnio.Dprint(6, 'transport_cspace.online response: %s' % str(result))
        else:
            dhnio.Dprint(6, "transport_cspace.online ERROR service not started")
    
    def offline(self):
        server = self._getserver()
        if server is not None:
            try:
                result = server.offline()
            except:
                dhnio.DprintException()
                return
            dhnio.Dprint(6, 'transport_cspace.offline response: %s' % str(result))
        else:
            dhnio.Dprint(6, "transport_cspace.offline ERROR service not started")

    def add(self, buddy, keyid):
        server = self._getserver()
        if server is not None:
            try:
                result = server.add(buddy, keyid)
            except:
                dhnio.DprintException()
                return
            dhnio.Dprint(6, 'transport_cspace.add response: %s' % str(result))
        else:
            dhnio.Dprint(6, "transport_cspace.add ERROR service not started")

    def remove(self, buddy):
        server = self._getserver()
        if server is not None:
            try:
                result = server.remove(buddy)
            except:
                dhnio.DprintException()
                return
            dhnio.Dprint(6, 'transport_cspace.online response: %s' % str(result))
        else:
            dhnio.Dprint(6, "transport_cspace.remove ERROR service not started")

#    def list(self):
#        server = self._getserver()
#        if server is not None:
#            buddies = server.list()
#            if buddies[0] is False:
#                print "Not connected."
#                return
#            for buddystatus in buddies:
#                print "%s: (%i) %s" % tuple(buddystatus)
#        else:
#            dhnio.Dprint(6, "transport_cspace.list ERROR service not started")

    def probe(self, buddy):
        server = self._getserver()
        if server is not None:
            try:
                result = server.probe(buddy)
            except:
                dhnio.DprintException()
                return
            dhnio.Dprint(6, 'transport_cspace.online response: %s' % str(result))
        else:
            dhnio.Dprint(6, "transport_cspace.probe ERROR service not started")

    def send(self, keyID, filename, transferID, description):
        server = self._getserver()
        if server is not None:
            try:
                server.send(str(keyID), filename, transferID, description)
            except:
                dhnio.DprintException()
                return
        else:
            dhnio.Dprint(6, "transport_cspace.send ERROR service not started")

    def cancel(self, transferID):
        server = self._getserver()
        if server is not None:
            try:
                server.cancel(transferID)
            except:
                dhnio.DprintException()
                return
        else:
            dhnio.Dprint(6, "transport_cspace.cancel ERROR service not started")
        
    def set_receiving_streams(self, num):
        server = self._getserver()
        if server is not None:
            try:
                server.set_receiving_streams(num)
            except:
                dhnio.DprintException()
                return
        else:
            dhnio.Dprint(6, "transport_cspace.set_receiving_streams ERROR service not started")

    def set_sending_streams(self, num):
        server = self._getserver()
        if server is not None:
            try:
                server.set_sending_streams(num)
            except:
                dhnio.DprintException()
                return
        else:
            dhnio.Dprint(6, "transport_cspace.set_sending_streams ERROR service not started")

    def stop(self):
        server = self._getserver()
        if server is not None:
            try:
                server.stop()
                dhnio.Dprint(6, "transport_cspace.stop service stopped")
            except:
                pass
        
#------------------------------------------------------------------------------ 

class CSpaceServiceProtocol(protocol.ProcessProtocol):
    def __init__(self, process_finished_callback=None):
        self.process_finished_callback = process_finished_callback
    def outReceived(self, inp):
        for line in inp.splitlines():
            try:
                command = line.split(' ')[0]
                data = str(line[len(command)+1:].strip())
                if data:
                    try:
                        data = eval(data)
                    except:
                        pass
            except:
                dhnio.Dprint(2, 'transport_cspace.CSpaceServiceProtocol.outReceived ERROR: %s' % line)
                dhnio.DprintException()
            # dhnio.Dprint(6, 'transport_cspace.CSpaceServiceProtocol.outReceived %s %s' % (command, str(data)))
            callback(command.lstrip('[').rstrip(']'), data)
    def errReceived(self, inp):
        for line in inp.splitlines():
            dhnio.Dprint(2, line)
    def processEnded(self, reason):
        if self.process_finished_callback:
            self.process_finished_callback(reason)
    
#------------------------------------------------------------------------------ 

class PseudoListener:
    implements(interfaces.IListeningPort)
    def startListening(self):
        A('online')
        return succeed(A().state)
    def stopListening(self):
        return shutdown()
    def getHost(self):
        return keyID()

def getListener():
    global _Listener
    if _Listener is None:
        _Listener = PseudoListener()
    return _Listener

#------------------------------------------------------------------------------ 

def SendStatusFuncDefault(host, filename, status, proto='', error=None, message=''):
    try:
        from transport_control import sendStatusReport
        sendStatusReport(host, filename, status, proto, error, message)
    except:
        dhnio.DprintException()

def ReceiveStatusFuncDefault(filename, status, proto='', host=None, error=None, message=''):
    try:
        from transport_control import receiveStatusReport
        receiveStatusReport(filename, status, proto, host, error, message)
    except:
        dhnio.DprintException()

def RegisterTransferFuncDefault(send_or_receive, remote_address, callback, filename, size, description=''):
    try:
        from transport_control import register_transfer
        return register_transfer('cspace', send_or_receive, remote_address, callback, filename, size, description)
    except:
        dhnio.DprintException()
        return None
    
def UnRegisterTransferFuncDefault(transfer_id):
    try:
        from transport_control import unregister_transfer
        return unregister_transfer(transfer_id)
    except:
        dhnio.DprintException()
    return None

_SendStatusFunc = SendStatusFuncDefault
_ReceiveStatusFunc = ReceiveStatusFuncDefault
_RegisterTransferFunc = RegisterTransferFuncDefault
_UnRegisterTransferFunc = UnRegisterTransferFuncDefault

def SetSendStatusFunc(f):
    global _SendStatusFunc
    _SendStatusFunc = f

def SetReceiveStatusFunc(f):
    global _ReceiveStatusFunc
    _ReceiveStatusFunc = f

def SetRegisterTransferFunc(f):
    global _RegisterTransferFunc
    _RegisterTransferFunc = f
    
def SetUnRegisterTransferFunc(f):
    global _UnRegisterTransferFunc
    _UnRegisterTransferFunc = f
    
def SetContactStatusNotifyFunc(f):
    global _ContactStatusNotifyFunc
    _ContactStatusNotifyFunc = f

#------------------------------------------------------------------------------ 

def parseCommandLine():
    oparser = optparse.OptionParser()
    oparser.add_option("-s", "--send", dest="send", action="store_true", help='send a file to remote peer')
    oparser.add_option("-r", "--receive", dest="receive", action="store_true", help='waiting files from other users')
    oparser.add_option("-b", "--basedir", dest="basedir", help="base directory location where dhnet.py is located")
    oparser.add_option("-d", "--debug", dest="debug", action="store_true", help="also print CSpace output")
    oparser.set_default('debug', False)
    oparser.set_default('send', False)
    oparser.set_default('receive', False)
    oparser.set_default('basedir', '.')
    (options, args) = oparser.parse_args()
    return options, args

def main():
    global _BaseDir
    global _Debug
    options, args = parseCommandLine()
    _BaseDir = options.basedir 
    _Debug = options.debug
    dhnio.init()
    dhnio.SetDebug(18)
    settings.init()
    import identitycache
    identitycache.init()
    reactor.addSystemEventTrigger('before', 'shutdown', shutdown_final)
#    from transport_control import _InitDone
#    _InitDone = True
    def initDone(state, options, args):
        if state != 'online':
            print 'state is %s, exit' % state
            reactor.stop()
            return
        if options.send:
            def _random_sending(count):
                import random
                if count > 10:
                    return
                print 'sending file %s to %s' % (args[0], args[1])
                reactor.callLater(random.randint(0, 2), _random_sending, count+1)
                send(args[0], args[1])
                count += 1
            reactor.callLater(0.5, _random_sending, 0)
            return
        if options.receive:
            print 'state is %s, receiving...' % state
            return
        print 'ONLINE !!!'
    init().addCallback(initDone, options, args)
    # reactor.callLater(20, A, 'shutdown')
    reactor.run() 
    
#------------------------------------------------------------------------------ 

if __name__ == '__main__':
    main()
